@@ -1,5 +1,5 @@
 # Redmine - project management software
-# Copyright (C) 2006-2014  Jean-Philippe Lang
+# Copyright (C) 2006-2016  Jean-Philippe Lang
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -18,6 +18,10 @@
 class MailHandlerController < ActionController::Base
   before_filter :check_credential
 
+  # Displays the email submission form
+  def new
+  end
+
   # Submits an incoming email to MailHandler
   def index
     options = params.dup
