@@ -1,5 +1,5 @@
 # Redmine - project management software
-# Copyright (C) 2006-2014  Jean-Philippe Lang
+# Copyright (C) 2006-2016  Jean-Philippe Lang
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -58,7 +58,7 @@ class NewsController < ApplicationController
   end
 
   def show
-    @comments = @news.comments
+    @comments = @news.comments.to_a
     @comments.reverse! if User.current.wants_comments_in_reverse_order?
   end
 
