@@ -1,5 +1,5 @@
 # Redmine - project management software
-# Copyright (C) 2006-2014  Jean-Philippe Lang
+# Copyright (C) 2006-2016  Jean-Philippe Lang
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -17,7 +17,6 @@
 
 class ProjectsController < ApplicationController
   menu_item :overview
-  menu_item :roadmap, :only => :roadmap
   menu_item :settings, :only => :settings
 
   before_filter :find_project, :except => [ :index, :list, :new, :create, :copy ]
@@ -26,6 +25,7 @@ class ProjectsController < ApplicationController
   before_filter :require_admin, :only => [ :copy, :archive, :unarchive, :destroy ]
   accept_rss_auth :index
   accept_api_auth :index, :show, :create, :update, :destroy
+  require_sudo_mode :destroy
 
   after_filter :only => [:create, :edit, :update, :archive, :unarchive, :destroy] do |controller|
     if controller.request.post?
@@ -33,35 +33,30 @@ class ProjectsController < ApplicationController
     end
   end
 
-  helper :sort
-  include SortHelper
   helper :custom_fields
-  include CustomFieldsHelper
   helper :issues
   helper :queries
-  include QueriesHelper
   helper :repositories
-  include RepositoriesHelper
-  include ProjectsHelper
   helper :members
 
   # Lists visible projects
   def index
+    scope = Project.visible.sorted
+
     respond_to do |format|
       format.html {
-        scope = Project
         unless params[:closed]
           scope = scope.active
         end
-        @projects = scope.visible.order('lft').to_a
+        @projects = scope.to_a
       }
       format.api  {
         @offset, @limit = api_offset_and_limit
-        @project_count = Project.visible.count
-        @projects = Project.visible.offset(@offset).limit(@limit).order('lft').to_a
+        @project_count = scope.count
+        @projects = scope.offset(@offset).limit(@limit).to_a
       }
       format.atom {
-        projects = Project.visible.order('created_on DESC').limit(Setting.feeds_limit.to_i).to_a
+        projects = scope.reorder(:created_on => :desc).limit(Setting.feeds_limit.to_i).to_a
         render_feed(projects, :title => "#{Setting.app_title}: #{l(:label_project_latest)}")
       }
     end
@@ -80,13 +75,9 @@ class ProjectsController < ApplicationController
     @project = Project.new
     @project.safe_attributes = params[:project]
 
-    if validate_parent_id && @project.save
-      @project.set_allowed_parent!(params[:project]['parent_id']) if params[:project].has_key?('parent_id')
-      # Add current user as a project member if current user is not admin
+    if @project.save
       unless User.current.admin?
-        r = Role.givable.find_by_id(Setting.new_project_user_role_id.to_i) || Role.givable.first
-        m = Member.new(:user => User.current, :roles => [r])
-        @project.members << m
+        @project.add_default_member(User.current)
       end
       respond_to do |format|
         format.html {
@@ -119,8 +110,7 @@ class ProjectsController < ApplicationController
       Mailer.with_deliveries(params[:notifications] == '1') do
         @project = Project.new
         @project.safe_attributes = params[:project]
-        if validate_parent_id && @project.copy(@source_project, :only => params[:only])
-          @project.set_allowed_parent!(params[:project]['parent_id']) if params[:project].has_key?('parent_id')
+        if @project.copy(@source_project, :only => params[:only])
           flash[:notice] = l(:notice_successful_create)
           redirect_to settings_project_path(@project)
         elsif !@project.new_record?
@@ -154,7 +144,7 @@ class ProjectsController < ApplicationController
     @open_issues_by_tracker = Issue.visible.open.where(cond).group(:tracker).count
     @total_issues_by_tracker = Issue.visible.where(cond).group(:tracker).count
 
-    if User.current.allowed_to?(:view_time_entries, @project)
+    if User.current.allowed_to_view_all_time_entries?(@project)
       @total_hours = TimeEntry.visible.where(cond).sum(:hours).to_f
     end
 
@@ -171,7 +161,7 @@ class ProjectsController < ApplicationController
     @issue_category ||= IssueCategory.new
     @member ||= @project.members.new
     @trackers = Tracker.sorted.to_a
-    @wiki ||= @project.wiki
+    @wiki ||= @project.wiki || Wiki.new(:project => @project)
   end
 
   def edit
@@ -179,8 +169,7 @@ class ProjectsController < ApplicationController
 
   def update
     @project.safe_attributes = params[:project]
-    if validate_parent_id && @project.save
-      @project.set_allowed_parent!(params[:project]['parent_id']) if params[:project].has_key?('parent_id')
+    if @project.save
       respond_to do |format|
         format.html {
           flash[:notice] = l(:notice_successful_update)
@@ -206,16 +195,16 @@ class ProjectsController < ApplicationController
   end
 
   def archive
-    if request.post?
-      unless @project.archive
-        flash[:error] = l(:error_can_not_archive_project)
-      end
+    unless @project.archive
+      flash[:error] = l(:error_can_not_archive_project)
     end
     redirect_to admin_projects_path(:status => params[:status])
   end
 
   def unarchive
-    @project.unarchive if request.post? && !@project.active?
+    unless @project.active?
+      @project.unarchive
+    end
     redirect_to admin_projects_path(:status => params[:status])
   end
 
@@ -242,21 +231,4 @@ class ProjectsController < ApplicationController
     # hide project in layout
     @project = nil
   end
-
-  private
-
-  # Validates parent_id param according to user's permissions
-  # TODO: move it to Project model in a validation that depends on User.current
-  def validate_parent_id
-    return true if User.current.admin?
-    parent_id = params[:project] && params[:project][:parent_id]
-    if parent_id || @project.new_record?
-      parent = parent_id.blank? ? nil : Project.find_by_id(parent_id.to_i)
-      unless @project.allowed_parents.include?(parent)
-        @project.errors.add :parent_id, :invalid
-        return false
-      end
-    end
-    true
-  end
 end
