@@ -1,5 +1,5 @@
 # Redmine - project management software
-# Copyright (C) 2006-2014  Jean-Philippe Lang
+# Copyright (C) 2006-2016  Jean-Philippe Lang
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -23,6 +23,8 @@ class RolesController < ApplicationController
   before_filter :find_role, :only => [:show, :edit, :update, :destroy]
   accept_api_auth :index, :show
 
+  require_sudo_mode :create, :update, :destroy
+
   def index
     respond_to do |format|
       format.html {
@@ -71,7 +73,7 @@ class RolesController < ApplicationController
   def update
     if @role.update_attributes(params[:role])
       flash[:notice] = l(:notice_successful_update)
-      redirect_to roles_path
+      redirect_to roles_path(:page => params[:page])
     else
       render :action => 'edit'
     end
