@@ -1,5 +1,5 @@
 # Redmine - project management software
-# Copyright (C) 2006-2014  Jean-Philippe Lang
+# Copyright (C) 2006-2016  Jean-Philippe Lang
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -21,7 +21,9 @@ class WorkflowsController < ApplicationController
   before_filter :require_admin
 
   def index
-    @workflow_counts = WorkflowTransition.count_by_tracker_and_role
+    @roles = Role.sorted.select(&:consider_workflow?)
+    @trackers = Tracker.sorted
+    @workflow_counts = WorkflowTransition.group(:tracker_id, :role_id).count
   end
 
   def edit
@@ -41,7 +43,9 @@ class WorkflowsController < ApplicationController
     end
 
     if @trackers && @roles && @statuses.any?
-      workflows = WorkflowTransition.where(:role_id => @roles.map(&:id), :tracker_id => @trackers.map(&:id))
+      workflows = WorkflowTransition.
+        where(:role_id => @roles.map(&:id), :tracker_id => @trackers.map(&:id)).
+        preload(:old_status, :new_status)
       @workflows = {}
       @workflows['always'] = workflows.select {|w| !w.author && !w.assignee}
       @workflows['author'] = workflows.select {|w| w.author}
@@ -72,7 +76,7 @@ class WorkflowsController < ApplicationController
   end
 
   def copy
-    @roles = Role.sorted
+    @roles = Role.sorted.select(&:consider_workflow?)
     @trackers = Tracker.sorted
 
     if params[:source_tracker_id].blank? || params[:source_tracker_id] == 'any'
