@@ -1,7 +1,7 @@
 # encoding: utf-8
 #
 # Redmine - project management software
-# Copyright (C) 2006-2014  Jean-Philippe Lang
+# Copyright (C) 2006-2016  Jean-Philippe Lang
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -25,6 +25,9 @@ module ApplicationHelper
   include Redmine::I18n
   include GravatarHelper::PublicMethods
   include Redmine::Pagination::Helper
+  include Redmine::SudoMode::Helper
+  include Redmine::Themes::Helper
+  include Redmine::Hook::Helper
 
   extend Forwardable
   def_delegators :wiki_helper, :wikitoolbar_for, :heads_for_wiki_formatter
@@ -80,7 +83,7 @@ module ApplicationHelper
       end
     end
     only_path = options[:only_path].nil? ? true : options[:only_path]
-    s = link_to(text, issue_path(issue, :only_path => only_path),
+    s = link_to(text, issue_url(issue, :only_path => only_path),
                 :class => issue.css_classes, :title => title)
     s << h(": #{subject}") if subject
     s = h("#{issue.project} - ") + s if options[:project]
@@ -93,8 +96,9 @@ module ApplicationHelper
   # * :download - Force download (default: false)
   def link_to_attachment(attachment, options={})
     text = options.delete(:text) || attachment.filename
-    route_method = options.delete(:download) ? :download_named_attachment_path : :named_attachment_path
+    route_method = options.delete(:download) ? :download_named_attachment_url : :named_attachment_url
     html_options = options.slice!(:only_path)
+    options[:only_path] = true unless options.key?(:only_path)
     url = send(route_method, attachment, attachment.filename, options)
     link_to text, url, html_options
   end
@@ -111,7 +115,8 @@ module ApplicationHelper
     link_to(
         h(text),
         {:controller => 'repositories', :action => 'revision', :id => repository.project, :repository_id => repository.identifier_param, :rev => rev},
-        :title => l(:label_revision_id, format_revision(revision))
+        :title => l(:label_revision_id, format_revision(revision)),
+        :accesskey => options[:accesskey]
       )
   end
 
@@ -119,9 +124,10 @@ module ApplicationHelper
   def link_to_message(message, options={}, html_options = nil)
     link_to(
       message.subject.truncate(60),
-      board_message_path(message.board_id, message.parent_id || message.id, {
+      board_message_url(message.board_id, message.parent_id || message.id, {
         :r => (message.parent_id && message.id),
-        :anchor => (message.parent_id ? "message-#{message.id}" : nil)
+        :anchor => (message.parent_id ? "message-#{message.id}" : nil),
+        :only_path => true
       }.merge(options)),
       html_options
     )
@@ -137,10 +143,10 @@ module ApplicationHelper
   def link_to_project(project, options={}, html_options = nil)
     if project.archived?
       h(project.name)
-    elsif options.key?(:action)
-      raise "#link_to_project no longer accepts :action option in Redmine 3.0"
     else
-      link_to project.name, project_path(project, options), html_options
+      link_to project.name,
+        project_url(project, {:only_path => true}.merge(options)),
+        html_options
     end
   end
 
@@ -155,6 +161,13 @@ module ApplicationHelper
     end
   end
 
+  # Generates a link to a version
+  def link_to_version(version, options = {})
+    return '' unless version && version.is_a?(Version)
+    options = {:title => format_date(version.effective_date)}.merge(options)
+    link_to_if version.visible?, format_version_name(version), version_path(version), options
+  end
+
   # Helper that formats object for html or text rendering
   def format_object(object, html=true, &block)
     if block_given?
@@ -176,7 +189,7 @@ module ApplicationHelper
     when 'Project'
       html ? link_to_project(object) : object.to_s
     when 'Version'
-      html ? link_to(object.name, version_path(object)) : object.to_s
+      html ? link_to_version(object) : object.to_s
     when 'TrueClass'
       l(:general_text_Yes)
     when 'FalseClass'
@@ -216,14 +229,6 @@ module ApplicationHelper
     link_to(name, "#", :onclick => onclick)
   end
 
-  def image_to_function(name, function, html_options = {})
-    html_options.symbolize_keys!
-    tag(:input, html_options.merge({
-        :type => "image", :src => image_path(name),
-        :onclick => (html_options[:onclick] ? "#{html_options[:onclick]}; " : "") + "#{function};"
-        }))
-  end
-
   def format_activity_title(text)
     h(truncate_single_line_raw(text, 100))
   end
@@ -254,7 +259,7 @@ module ApplicationHelper
   # Renders a tree of projects as a nested set of unordered lists
   # The given collection may be a subset of the whole project tree
   # (eg. some intermediate nodes are private and can not be seen)
-  def render_project_nested_lists(projects)
+  def render_project_nested_lists(projects, &block)
     s = ''
     if projects.any?
       ancestors = []
@@ -274,7 +279,7 @@ module ApplicationHelper
         end
         classes = (ancestors.empty? ? 'root' : 'child')
         s << "<li class='#{classes}'><div class='#{classes}'>"
-        s << h(block_given? ? yield(project) : project.name)
+        s << h(block_given? ? capture(project, &block) : project.name)
         s << "</div>\n"
         ancestors << project
       end
@@ -325,7 +330,7 @@ module ApplicationHelper
   # Renders the project quick-jump box
   def render_project_jump_box
     return unless User.current.logged?
-    projects = User.current.memberships.collect(&:project).compact.select(&:active?).uniq
+    projects = User.current.projects.active.select(:id, :name, :identifier, :lft, :rgt).to_a
     if projects.any?
       options =
         ("<option value=''>#{ l(:label_jump_to_a_project) }</option>" +
@@ -335,12 +340,19 @@ module ApplicationHelper
         { :value => project_path(:id => p, :jump => current_menu_item) }
       end
 
+      content_tag( :span, nil, :class => 'jump-box-arrow') +
       select_tag('project_quick_jump_box', options, :onchange => 'if (this.value != \'\') { window.location = this.value; }')
     end
   end
 
   def project_tree_options_for_select(projects, options = {})
-    s = ''
+    s = ''.html_safe
+    if blank_text = options[:include_blank]
+      if blank_text == true
+        blank_text = '&nbsp;'.html_safe
+      end
+      s << content_tag('option', blank_text, :value => '')
+    end
     project_tree(projects) do |project, level|
       name_prefix = (level > 0 ? '&nbsp;' * 2 * level + '&#187; ' : '').html_safe
       tag_options = {:value => project.id}
@@ -387,30 +399,12 @@ module ApplicationHelper
     s.html_safe
   end
 
-  # Options for the new membership projects combo-box
-  def options_for_membership_project_select(principal, projects)
-    options = content_tag('option', "--- #{l(:actionview_instancetag_blank_option)} ---")
-    options << project_tree_options_for_select(projects) do |p|
-      {:disabled => principal.projects.to_a.include?(p)}
-    end
-    options
-  end
-
   def option_tag(name, text, value, selected=nil, options={})
     content_tag 'option', value, options.merge(:value => value, :selected => (value == selected))
   end
 
-  # Truncates and returns the string as a single line
-  def truncate_single_line(string, *args)
-    ActiveSupport::Deprecation.warn(
-      "ApplicationHelper#truncate_single_line is deprecated and will be removed in Rails 4 poring")
-    # Rails 4 ActionView::Helpers::TextHelper#truncate escapes.
-    # So, result is broken.
-    truncate(string.to_s, *args).gsub(%r{[\r\n]+}m, ' ')
-  end
-
   def truncate_single_line_raw(string, length)
-    string.truncate(length).gsub(%r{[\r\n]+}m, ' ')
+    string.to_s.truncate(length).gsub(%r{[\r\n]+}m, ' ')
   end
 
   # Truncates at line break after 250 characters or options[:length]
@@ -438,7 +432,7 @@ module ApplicationHelper
   def time_tag(time)
     text = distance_of_time_in_words(Time.now, time)
     if @project
-      link_to(text, {:controller => 'activities', :action => 'index', :id => @project, :from => User.current.time_to_date(time)}, :title => format_time(time))
+      link_to(text, project_activity_path(@project, :from => User.current.time_to_date(time)), :title => format_time(time))
     else
       content_tag('abbr', text, :title => format_time(time))
     end
@@ -577,7 +571,7 @@ module ApplicationHelper
     end
     return '' if text.blank?
     project = options[:project] || @project || (obj && obj.respond_to?(:project) ? obj.project : nil)
-    only_path = options.delete(:only_path) == false ? false : true
+    @only_path = only_path = options.delete(:only_path) == false ? false : true
 
     text = text.dup
     macros = catch_macros(text)
@@ -618,7 +612,7 @@ module ApplicationHelper
       parsed << text
       if tag
         if closing
-          if tags.last == tag.downcase
+          if tags.last && tags.last.casecmp(tag) == 0
             tags.pop
           end
         else
@@ -635,6 +629,8 @@ module ApplicationHelper
   end
 
   def parse_inline_attachments(text, project, obj, attr, only_path, options)
+    return if options[:inline_attachments] == false
+
     # when using an image link, try to use an attachment, if possible
     attachments = options[:attachments] || []
     attachments += obj.attachments if obj.respond_to?(:attachments)
@@ -642,8 +638,8 @@ module ApplicationHelper
       text.gsub!(/src="([^\/"]+\.(bmp|gif|jpg|jpe|jpeg|png))"(\s+alt="([^"]*)")?/i) do |m|
         filename, ext, alt, alttext = $1.downcase, $2, $3, $4
         # search for the picture in attachments
-        if found = Attachment.latest_attach(attachments, filename)
-          image_url = download_named_attachment_path(found, found.filename, :only_path => only_path)
+        if found = Attachment.latest_attach(attachments, CGI.unescape(filename))
+          image_url = download_named_attachment_url(found, found.filename, :only_path => only_path)
           desc = found.description.to_s.gsub('"', '')
           if !desc.blank? && alttext.blank?
             alt = " title=\"#{desc}\" alt=\"#{desc}\""
@@ -746,144 +742,139 @@ module ApplicationHelper
   #     identifier:version:1.0.0
   #     identifier:source:some/file
   def parse_redmine_links(text, default_project, obj, attr, only_path, options)
-    text.gsub!(%r{([\s\(,\-\[\>]|^)(!)?(([a-z0-9\-_]+):)?(attachment|document|version|forum|news|message|project|commit|source|export)?(((#)|((([a-z0-9\-_]+)\|)?(r)))((\d+)((#note)?-(\d+))?)|(:)([^"\s<>][^\s<>]*?|"[^"]+?"))(?=(?=[[:punct:]][^A-Za-z0-9_/])|,|\s|\]|<|$)}) do |m|
-      leading, esc, project_prefix, project_identifier, prefix, repo_prefix, repo_identifier, sep, identifier, comment_suffix, comment_id = $1, $2, $3, $4, $5, $10, $11, $8 || $12 || $18, $14 || $19, $15, $17
-      link = nil
-      project = default_project
-      if project_identifier
-        project = Project.visible.find_by_identifier(project_identifier)
-      end
-      if esc.nil?
-        if prefix.nil? && sep == 'r'
-          if project
-            repository = nil
-            if repo_identifier
-              repository = project.repositories.detect {|repo| repo.identifier == repo_identifier}
-            else
-              repository = project.repository
-            end
-            # project.changesets.visible raises an SQL error because of a double join on repositories
-            if repository &&
-                 (changeset = Changeset.visible.
-                                  find_by_repository_id_and_revision(repository.id, identifier))
-              link = link_to(h("#{project_prefix}#{repo_prefix}r#{identifier}"),
-                             {:only_path => only_path, :controller => 'repositories',
-                              :action => 'revision', :id => project,
-                              :repository_id => repository.identifier_param,
-                              :rev => changeset.revision},
-                             :class => 'changeset',
-                             :title => truncate_single_line_raw(changeset.comments, 100))
-            end
-          end
-        elsif sep == '#'
-          oid = identifier.to_i
-          case prefix
-          when nil
-            if oid.to_s == identifier &&
-                  issue = Issue.visible.includes(:status).find_by_id(oid)
-              anchor = comment_id ? "note-#{comment_id}" : nil
-              link = link_to(h("##{oid}#{comment_suffix}"),
-                             {:only_path => only_path, :controller => 'issues',
-                              :action => 'show', :id => oid, :anchor => anchor},
-                             :class => issue.css_classes,
-                             :title => "#{issue.subject.truncate(100)} (#{issue.status.name})")
-            end
-          when 'document'
-            if document = Document.visible.find_by_id(oid)
-              link = link_to h(document.title), {:only_path => only_path, :controller => 'documents', :action => 'show', :id => document},
-                                                :class => 'document'
-            end
-          when 'version'
-            if version = Version.visible.find_by_id(oid)
-              link = link_to h(version.name), {:only_path => only_path, :controller => 'versions', :action => 'show', :id => version},
-                                              :class => 'version'
-            end
-          when 'message'
-            if message = Message.visible.includes(:parent).find_by_id(oid)
-              link = link_to_message(message, {:only_path => only_path}, :class => 'message')
-            end
-          when 'forum'
-            if board = Board.visible.find_by_id(oid)
-              link = link_to h(board.name), {:only_path => only_path, :controller => 'boards', :action => 'show', :id => board, :project_id => board.project},
-                                             :class => 'board'
-            end
-          when 'news'
-            if news = News.visible.find_by_id(oid)
-              link = link_to h(news.title), {:only_path => only_path, :controller => 'news', :action => 'show', :id => news},
-                                            :class => 'news'
-            end
-          when 'project'
-            if p = Project.visible.find_by_id(oid)
-              link = link_to_project(p, {:only_path => only_path}, :class => 'project')
-            end
-          end
-        elsif sep == ':'
-          # removes the double quotes if any
-          name = identifier.gsub(%r{^"(.*)"$}, "\\1")
-          name = CGI.unescapeHTML(name)
-          case prefix
-          when 'document'
-            if project && document = project.documents.visible.find_by_title(name)
-              link = link_to h(document.title), {:only_path => only_path, :controller => 'documents', :action => 'show', :id => document},
-                                                :class => 'document'
-            end
-          when 'version'
-            if project && version = project.versions.visible.find_by_name(name)
-              link = link_to h(version.name), {:only_path => only_path, :controller => 'versions', :action => 'show', :id => version},
-                                              :class => 'version'
-            end
-          when 'forum'
-            if project && board = project.boards.visible.find_by_name(name)
-              link = link_to h(board.name), {:only_path => only_path, :controller => 'boards', :action => 'show', :id => board, :project_id => board.project},
-                                             :class => 'board'
-            end
-          when 'news'
-            if project && news = project.news.visible.find_by_title(name)
-              link = link_to h(news.title), {:only_path => only_path, :controller => 'news', :action => 'show', :id => news},
-                                            :class => 'news'
-            end
-          when 'commit', 'source', 'export'
+    text.gsub!(%r{<a( [^>]+?)?>(.*?)</a>|([\s\(,\-\[\>]|^)(!)?(([a-z0-9\-_]+):)?(attachment|document|version|forum|news|message|project|commit|source|export)?(((#)|((([a-z0-9\-_]+)\|)?(r)))((\d+)((#note)?-(\d+))?)|(:)([^"\s<>][^\s<>]*?|"[^"]+?"))(?=(?=[[:punct:]][^A-Za-z0-9_/])|,|\s|\]|<|$)}) do |m|
+      tag_content, leading, esc, project_prefix, project_identifier, prefix, repo_prefix, repo_identifier, sep, identifier, comment_suffix, comment_id = $2, $3, $4, $5, $6, $7, $12, $13, $10 || $14 || $20, $16 || $21, $17, $19
+      if tag_content
+        $&
+      else
+        link = nil
+        project = default_project
+        if project_identifier
+          project = Project.visible.find_by_identifier(project_identifier)
+        end
+        if esc.nil?
+          if prefix.nil? && sep == 'r'
             if project
               repository = nil
-              if name =~ %r{^(([a-z0-9\-_]+)\|)(.+)$}
-                repo_prefix, repo_identifier, name = $1, $2, $3
+              if repo_identifier
                 repository = project.repositories.detect {|repo| repo.identifier == repo_identifier}
               else
                 repository = project.repository
               end
-              if prefix == 'commit'
-                if repository && (changeset = Changeset.visible.where("repository_id = ? AND scmid LIKE ?", repository.id, "#{name}%").first)
-                  link = link_to h("#{project_prefix}#{repo_prefix}#{name}"), {:only_path => only_path, :controller => 'repositories', :action => 'revision', :id => project, :repository_id => repository.identifier_param, :rev => changeset.identifier},
-                                               :class => 'changeset',
-                                               :title => truncate_single_line_raw(changeset.comments, 100)
-                end
-              else
-                if repository && User.current.allowed_to?(:browse_repository, project)
-                  name =~ %r{^[/\\]*(.*?)(@([^/\\@]+?))?(#(L\d+))?$}
-                  path, rev, anchor = $1, $3, $5
-                  link = link_to h("#{project_prefix}#{prefix}:#{repo_prefix}#{name}"), {:only_path => only_path, :controller => 'repositories', :action => (prefix == 'export' ? 'raw' : 'entry'), :id => project, :repository_id => repository.identifier_param,
-                                                          :path => to_path_param(path),
-                                                          :rev => rev,
-                                                          :anchor => anchor},
-                                                         :class => (prefix == 'export' ? 'source download' : 'source')
-                end
+              # project.changesets.visible raises an SQL error because of a double join on repositories
+              if repository &&
+                   (changeset = Changeset.visible.
+                                    find_by_repository_id_and_revision(repository.id, identifier))
+                link = link_to(h("#{project_prefix}#{repo_prefix}r#{identifier}"),
+                               {:only_path => only_path, :controller => 'repositories',
+                                :action => 'revision', :id => project,
+                                :repository_id => repository.identifier_param,
+                                :rev => changeset.revision},
+                               :class => 'changeset',
+                               :title => truncate_single_line_raw(changeset.comments, 100))
               end
-              repo_prefix = nil
             end
-          when 'attachment'
-            attachments = options[:attachments] || []
-            attachments += obj.attachments if obj.respond_to?(:attachments)
-            if attachments && attachment = Attachment.latest_attach(attachments, name)
-              link = link_to_attachment(attachment, :only_path => only_path, :download => true, :class => 'attachment')
+          elsif sep == '#'
+            oid = identifier.to_i
+            case prefix
+            when nil
+              if oid.to_s == identifier &&
+                issue = Issue.visible.find_by_id(oid)
+                anchor = comment_id ? "note-#{comment_id}" : nil
+                link = link_to("##{oid}#{comment_suffix}",
+                               issue_url(issue, :only_path => only_path, :anchor => anchor),
+                               :class => issue.css_classes,
+                               :title => "#{issue.tracker.name}: #{issue.subject.truncate(100)} (#{issue.status.name})")
+              end
+            when 'document'
+              if document = Document.visible.find_by_id(oid)
+                link = link_to(document.title, document_url(document, :only_path => only_path), :class => 'document')
+              end
+            when 'version'
+              if version = Version.visible.find_by_id(oid)
+                link = link_to(version.name, version_url(version, :only_path => only_path), :class => 'version')
+              end
+            when 'message'
+              if message = Message.visible.find_by_id(oid)
+                link = link_to_message(message, {:only_path => only_path}, :class => 'message')
+              end
+            when 'forum'
+              if board = Board.visible.find_by_id(oid)
+                link = link_to(board.name, project_board_url(board.project, board, :only_path => only_path), :class => 'board')
+              end
+            when 'news'
+              if news = News.visible.find_by_id(oid)
+                link = link_to(news.title, news_url(news, :only_path => only_path), :class => 'news')
+              end
+            when 'project'
+              if p = Project.visible.find_by_id(oid)
+                link = link_to_project(p, {:only_path => only_path}, :class => 'project')
+              end
             end
-          when 'project'
-            if p = Project.visible.where("identifier = :s OR LOWER(name) = :s", :s => name.downcase).first
-              link = link_to_project(p, {:only_path => only_path}, :class => 'project')
+          elsif sep == ':'
+            # removes the double quotes if any
+            name = identifier.gsub(%r{^"(.*)"$}, "\\1")
+            name = CGI.unescapeHTML(name)
+            case prefix
+            when 'document'
+              if project && document = project.documents.visible.find_by_title(name)
+                link = link_to(document.title, document_url(document, :only_path => only_path), :class => 'document')
+              end
+            when 'version'
+              if project && version = project.versions.visible.find_by_name(name)
+                link = link_to(version.name, version_url(version, :only_path => only_path), :class => 'version')
+              end
+            when 'forum'
+              if project && board = project.boards.visible.find_by_name(name)
+                link = link_to(board.name, project_board_url(board.project, board, :only_path => only_path), :class => 'board')
+              end
+            when 'news'
+              if project && news = project.news.visible.find_by_title(name)
+                link = link_to(news.title, news_url(news, :only_path => only_path), :class => 'news')
+              end
+            when 'commit', 'source', 'export'
+              if project
+                repository = nil
+                if name =~ %r{^(([a-z0-9\-_]+)\|)(.+)$}
+                  repo_prefix, repo_identifier, name = $1, $2, $3
+                  repository = project.repositories.detect {|repo| repo.identifier == repo_identifier}
+                else
+                  repository = project.repository
+                end
+                if prefix == 'commit'
+                  if repository && (changeset = Changeset.visible.where("repository_id = ? AND scmid LIKE ?", repository.id, "#{name}%").first)
+                    link = link_to h("#{project_prefix}#{repo_prefix}#{name}"), {:only_path => only_path, :controller => 'repositories', :action => 'revision', :id => project, :repository_id => repository.identifier_param, :rev => changeset.identifier},
+                                                 :class => 'changeset',
+                                                 :title => truncate_single_line_raw(changeset.comments, 100)
+                  end
+                else
+                  if repository && User.current.allowed_to?(:browse_repository, project)
+                    name =~ %r{^[/\\]*(.*?)(@([^/\\@]+?))?(#(L\d+))?$}
+                    path, rev, anchor = $1, $3, $5
+                    link = link_to h("#{project_prefix}#{prefix}:#{repo_prefix}#{name}"), {:only_path => only_path, :controller => 'repositories', :action => (prefix == 'export' ? 'raw' : 'entry'), :id => project, :repository_id => repository.identifier_param,
+                                                            :path => to_path_param(path),
+                                                            :rev => rev,
+                                                            :anchor => anchor},
+                                                           :class => (prefix == 'export' ? 'source download' : 'source')
+                  end
+                end
+                repo_prefix = nil
+              end
+            when 'attachment'
+              attachments = options[:attachments] || []
+              attachments += obj.attachments if obj.respond_to?(:attachments)
+              if attachments && attachment = Attachment.latest_attach(attachments, name)
+                link = link_to_attachment(attachment, :only_path => only_path, :download => true, :class => 'attachment')
+              end
+            when 'project'
+              if p = Project.visible.where("identifier = :s OR LOWER(name) = :s", :s => name.downcase).first
+                link = link_to_project(p, {:only_path => only_path}, :class => 'project')
+              end
             end
           end
         end
+        (leading + (link || "#{project_prefix}#{prefix}#{repo_prefix}#{sep}#{identifier}#{comment_suffix}"))
       end
-      (leading + (link || "#{project_prefix}#{prefix}#{repo_prefix}#{sep}#{identifier}#{comment_suffix}"))
     end
   end
 
@@ -892,12 +883,12 @@ module ApplicationHelper
   def parse_sections(text, project, obj, attr, only_path, options)
     return unless options[:edit_section_links]
     text.gsub!(HEADING_RE) do
-      heading = $1
+      heading, level = $1, $2
       @current_section += 1
       if @current_section > 1
         content_tag('div',
           link_to(image_tag('edit.png'), options[:edit_section_links].merge(:section => @current_section)),
-          :class => 'contextual',
+          :class => "contextual heading-#{level}",
           :title => l(:button_edit_section),
           :id => "section-#{@current_section}") + heading.html_safe
       else
@@ -1029,11 +1020,6 @@ module ApplicationHelper
     (blank ? [["(auto)", ""]] : []) + languages_options
   end
 
-  def label_tag_for(name, option_tags = nil, options = {})
-    label_text = l(("field_"+field.to_s.gsub(/\_id$/, "")).to_sym) + (options.delete(:required) ? @template.content_tag("span", " *", :class => "required"): "")
-    content_tag("label", label_text)
-  end
-
   def labelled_form_for(*args, &proc)
     args << {} unless args.last.is_a?(Hash)
     options = args.last
@@ -1051,14 +1037,6 @@ module ApplicationHelper
     fields_for(*args, &proc)
   end
 
-  def labelled_remote_form_for(*args, &proc)
-    ActiveSupport::Deprecation.warn "ApplicationHelper#labelled_remote_form_for is deprecated and will be removed in Redmine 2.2."
-    args << {} unless args.last.is_a?(Hash)
-    options = args.last
-    options.merge!({:builder => Redmine::Views::LabelledFormBuilder, :remote => true})
-    form_for(*args, &proc)
-  end
-
   def error_messages_for(*objects)
     html = ""
     objects = objects.map {|o| o.is_a?(String) ? instance_variable_get("@#{o}") : o}.compact
@@ -1119,25 +1097,30 @@ module ApplicationHelper
     link_to_function(l(:button_uncheck_all), "checkAll('#{form_name}', false)")
   end
 
+  def toggle_checkboxes_link(selector)
+    link_to_function image_tag('toggle_check.png'),
+      "toggleCheckboxesBySelector('#{selector}')",
+      :title => "#{l(:button_check_all)} / #{l(:button_uncheck_all)}"
+  end
+
   def progress_bar(pcts, options={})
     pcts = [pcts, pcts] unless pcts.is_a?(Array)
     pcts = pcts.collect(&:round)
     pcts[1] = pcts[1] - pcts[0]
     pcts << (100 - pcts[1] - pcts[0])
-    width = options[:width] || '100px;'
     legend = options[:legend] || ''
     content_tag('table',
       content_tag('tr',
         (pcts[0] > 0 ? content_tag('td', '', :style => "width: #{pcts[0]}%;", :class => 'closed') : ''.html_safe) +
         (pcts[1] > 0 ? content_tag('td', '', :style => "width: #{pcts[1]}%;", :class => 'done') : ''.html_safe) +
         (pcts[2] > 0 ? content_tag('td', '', :style => "width: #{pcts[2]}%;", :class => 'todo') : ''.html_safe)
-      ), :class => "progress progress-#{pcts[0]}", :style => "width: #{width};").html_safe +
+      ), :class => "progress progress-#{pcts[0]}").html_safe +
       content_tag('p', legend, :class => 'percent').html_safe
   end
 
   def checked_image(checked=true)
     if checked
-      image_tag 'toggle_check.png'
+      @checked_image_tag ||= image_tag('toggle_check.png')
     end
   end
 
@@ -1159,12 +1142,12 @@ module ApplicationHelper
 
   def calendar_for(field_id)
     include_calendar_headers_tags
-    javascript_tag("$(function() { $('##{field_id}').datepicker(datepickerOptions); });")
+    javascript_tag("$(function() { $('##{field_id}').addClass('date').datepicker(datepickerOptions); });")
   end
 
   def include_calendar_headers_tags
     unless @calendar_headers_tags_included
-      tags = javascript_include_tag("datepicker")
+      tags = ''.html_safe
       @calendar_headers_tags_included = true
       content_for :header_tags do
         start_of_week = Setting.start_of_week
@@ -1181,7 +1164,7 @@ module ApplicationHelper
                      "beforeShow: beforeShowDatePicker};")
         jquery_locale = l('jquery.locale', :default => current_language.to_s)
         unless jquery_locale == 'en'
-          tags << javascript_include_tag("i18n/jquery.ui.datepicker-#{jquery_locale}.js")
+          tags << javascript_include_tag("i18n/datepicker-#{jquery_locale}.js")
         end
         tags
       end
@@ -1241,11 +1224,6 @@ module ApplicationHelper
     super *sources, options
   end
 
-  # TODO: remove this in 2.5.0
-  def has_content?(name)
-    content_for?(name)
-  end
-
   def sidebar_content?
     content_for?(:sidebar) || view_layouts_base_sidebar_hook_response.present?
   end
@@ -1275,13 +1253,21 @@ module ApplicationHelper
     end
   end
 
+  # Returns a link to edit user's avatar if avatars are enabled
+  def avatar_edit_link(user, options={})
+    if Setting.gravatar_enabled?
+      url = "https://gravatar.com"
+      link_to avatar(user, {:title => l(:button_edit)}.merge(options)), url, :target => '_blank'
+    end
+  end
+
   def sanitize_anchor_name(anchor)
     anchor.gsub(%r{[^\s\-\p{Word}]}, '').gsub(%r{\s+(\-+\s*)?}, '-')
   end
 
   # Returns the javascript tags that are included in the html layout head
   def javascript_heads
-    tags = javascript_include_tag('jquery-1.11.1-ui-1.11.0-ujs-3.1.1', 'application')
+    tags = javascript_include_tag('jquery-1.11.1-ui-1.11.0-ujs-3.1.4', 'application', 'responsive')
     unless User.current.pref.warn_on_leaving_unsaved == '0'
       tags << "\n".html_safe + javascript_tag("$(window).load(function(){ warnLeavingUnsaved('#{escape_javascript l(:text_warn_on_leaving_unsaved)}'); });")
     end
@@ -1332,6 +1318,11 @@ module ApplicationHelper
     end
   end
 
+  def generate_csv(&block)
+    decimal_separator = l(:general_csv_decimal_separator)
+    encoding = l(:general_csv_encoding)
+  end
+
   private
 
   def wiki_helper
