@@ -1,7 +1,7 @@
 # encoding: utf-8
 #
 # Redmine - project management software
-# Copyright (C) 2006-2014  Jean-Philippe Lang
+# Copyright (C) 2006-2016  Jean-Philippe Lang
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -21,14 +21,35 @@ module QueriesHelper
   include ApplicationHelper
 
   def filters_options_for_select(query)
-    options_for_select(filters_options(query))
-  end
-
-  def filters_options(query)
-    options = [[]]
-    options += query.available_filters.map do |field, field_options|
-      [field_options[:name], field]
+    ungrouped = []
+    grouped = {}
+    query.available_filters.map do |field, field_options|
+      if [:tree, :relation].include?(field_options[:type]) 
+        group = :label_related_issues
+      elsif field =~ /^(.+)\./
+        # association filters
+        group = "field_#{$1}"
+      elsif %w(member_of_group assigned_to_role).include?(field)
+        group = :field_assigned_to
+      elsif field_options[:type] == :date_past || field_options[:type] == :date
+        group = :label_date
+      end
+      if group
+        (grouped[group] ||= []) << [field_options[:name], field]
+      else
+        ungrouped << [field_options[:name], field]
+      end
     end
+    # Don't group dates if there's only one (eg. time entries filters)
+    if grouped[:label_date].try(:size) == 1 
+      ungrouped << grouped.delete(:label_date).first
+    end
+    s = options_for_select([[]] + ungrouped)
+    if grouped.present?
+      localized_grouped = grouped.map {|k,v| [l(k), v]}
+      s << grouped_options_for_select(localized_grouped)
+    end
+    s
   end
 
   def query_filters_hidden_tags(query)
@@ -63,6 +84,14 @@ module QueriesHelper
     tags
   end
 
+  def available_totalable_columns_tags(query)
+    tags = ''.html_safe
+    query.available_totalable_columns.each do |column|
+      tags << content_tag('label', check_box_tag('t[]', column.name.to_s, query.totalable_columns.include?(column), :id => nil) + " #{column.caption}", :class => 'inline')
+    end
+    tags
+  end
+
   def query_available_inline_columns_options(query)
     (query.available_inline_columns - query.columns).reject(&:frozen?).collect {|column| [column.caption, column.name]}
   end
@@ -76,6 +105,20 @@ module QueriesHelper
     render :partial => 'queries/columns', :locals => {:query => query, :tag_name => tag_name}
   end
 
+  def render_query_totals(query)
+    return unless query.totalable_columns.present?
+    totals = query.totalable_columns.map do |column|
+      total_tag(column, query.total_for(column))
+    end
+    content_tag('p', totals.join(" ").html_safe, :class => "query-totals")
+  end
+
+  def total_tag(column, value)
+    label = content_tag('span', "#{column.caption}:")
+    value = content_tag('span', format_object(value), :class => 'value')
+    content_tag('span', label + " " + value, :class => "total-for-#{column.name.to_s.dasherize}")
+  end
+
   def column_header(column)
     column.sortable ? sort_header_tag(column.name.to_s, :caption => column.caption,
                                                         :default_order => column.default_order) :
@@ -97,14 +140,15 @@ module QueriesHelper
       link_to value, issue_path(issue)
     when :subject
       link_to value, issue_path(issue)
+    when :parent
+      value ? (value.visible? ? link_to_issue(value, :subject => false) : "##{value.id}") : ''
     when :description
       issue.description? ? content_tag('div', textilizable(issue, :description), :class => "wiki") : ''
     when :done_ratio
-      progress_bar(value, :width => '80px')
+      progress_bar(value)
     when :relations
-      other = value.other_issue(issue)
       content_tag('span',
-        (l(value.label_for(issue)) + " " + link_to_issue(other, :subject => false, :tracker => false)).html_safe,
+        value.to_s(issue) {|other| link_to_issue(other, :subject => false, :tracker => false)}.html_safe,
         :class => value.css_classes_for(issue))
     else
       format_object(value)
@@ -120,14 +164,19 @@ module QueriesHelper
     end
   end
 
-  def csv_value(column, issue, value)
+  def csv_value(column, object, value)
     format_object(value, false) do |value|
       case value.class.name
       when 'Float'
         sprintf("%.2f", value).gsub('.', l(:general_csv_decimal_separator))
       when 'IssueRelation'
-        other = value.other_issue(issue)
-        l(value.label_for(issue)) + " ##{other.id}"
+        value.to_s(object)
+      when 'Issue'
+        if object.is_a?(TimeEntry)
+          "#{value.tracker} ##{value.id}: #{value.subject}"
+        else
+          value.id
+        end
       else
         value
       end
@@ -135,7 +184,7 @@ module QueriesHelper
   end
 
   def query_to_csv(items, query, options={})
-    encoding = l(:general_csv_encoding)
+    options ||= {}
     columns = (options[:columns] == 'all' ? query.available_inline_columns : query.inline_columns)
     query.available_block_columns.each do |column|
       if options[column.name].present?
@@ -143,15 +192,14 @@ module QueriesHelper
       end
     end
 
-    export = CSV.generate(:col_sep => l(:general_csv_separator)) do |csv|
+    Redmine::Export::CSV.generate do |csv|
       # csv header fields
-      csv << columns.collect {|c| Redmine::CodesetUtil.from_utf8(c.caption.to_s, encoding) }
+      csv << columns.map {|c| c.caption.to_s}
       # csv lines
       items.each do |item|
-        csv << columns.collect {|c| Redmine::CodesetUtil.from_utf8(csv_content(c, item), encoding) }
+        csv << columns.map {|c| csv_content(c, item)}
       end
     end
-    export
   end
 
   # Retrieve query from session or build a new query
@@ -169,12 +217,12 @@ module QueriesHelper
       @query = IssueQuery.new(:name => "_")
       @query.project = @project
       @query.build_from_params(params)
-      session[:query] = {:project_id => @query.project_id, :filters => @query.filters, :group_by => @query.group_by, :column_names => @query.column_names}
+      session[:query] = {:project_id => @query.project_id, :filters => @query.filters, :group_by => @query.group_by, :column_names => @query.column_names, :totalable_names => @query.totalable_names}
     else
       # retrieve from session
       @query = nil
       @query = IssueQuery.find_by_id(session[:query][:id]) if session[:query][:id]
-      @query ||= IssueQuery.new(:name => "_", :filters => session[:query][:filters], :group_by => session[:query][:group_by], :column_names => session[:query][:column_names])
+      @query ||= IssueQuery.new(:name => "_", :filters => session[:query][:filters], :group_by => session[:query][:group_by], :column_names => session[:query][:column_names], :totalable_names => session[:query][:totalable_names])
       @query.project = @project
     end
   end
@@ -185,7 +233,7 @@ module QueriesHelper
         @query = IssueQuery.find_by_id(session[:query][:id])
         return unless @query
       else
-        @query = IssueQuery.new(:name => "_", :filters => session[:query][:filters], :group_by => session[:query][:group_by], :column_names => session[:query][:column_names])
+        @query = IssueQuery.new(:name => "_", :filters => session[:query][:filters], :group_by => session[:query][:group_by], :column_names => session[:query][:column_names], :totalable_names => session[:query][:totalable_names])
       end
       if session[:query].has_key?(:project_id)
         @query.project_id = session[:query][:project_id]
@@ -195,4 +243,34 @@ module QueriesHelper
       @query
     end
   end
+
+  # Returns the query definition as hidden field tags
+  def query_as_hidden_field_tags(query)
+    tags = hidden_field_tag("set_filter", "1", :id => nil)
+
+    if query.filters.present?
+      query.filters.each do |field, filter|
+        tags << hidden_field_tag("f[]", field, :id => nil)
+        tags << hidden_field_tag("op[#{field}]", filter[:operator], :id => nil)
+        filter[:values].each do |value|
+          tags << hidden_field_tag("v[#{field}][]", value, :id => nil)
+        end
+      end
+    end
+    if query.column_names.present?
+      query.column_names.each do |name|
+        tags << hidden_field_tag("c[]", name, :id => nil)
+      end
+    end
+    if query.totalable_names.present?
+      query.totalable_names.each do |name|
+        tags << hidden_field_tag("t[]", name, :id => nil)
+      end
+    end
+    if query.group_by.present?
+      tags << hidden_field_tag("group_by", query.group_by, :id => nil)
+    end
+
+    tags
+  end
 end
