@@ -1,5 +1,5 @@
 # Redmine - project management software
-# Copyright (C) 2006-2014  Jean-Philippe Lang
+# Copyright (C) 2006-2016  Jean-Philippe Lang
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -18,9 +18,8 @@
 class Board < ActiveRecord::Base
   include Redmine::SafeAttributes
   belongs_to :project
-  has_many :topics, lambda {where("#{Message.table_name}.parent_id IS NULL").order("#{Message.table_name}.created_on DESC")}, :class_name => 'Message'
   has_many :messages, lambda {order("#{Message.table_name}.created_on DESC")}, :dependent => :destroy
-  belongs_to :last_message, :class_name => 'Message', :foreign_key => :last_message_id
+  belongs_to :last_message, :class_name => 'Message'
   acts_as_tree :dependent => :nullify
   acts_as_list :scope => '(project_id = #{project_id} AND parent_id #{parent_id ? "= #{parent_id}" : "IS NULL"})'
   acts_as_watchable
@@ -33,7 +32,6 @@ class Board < ActiveRecord::Base
 
   scope :visible, lambda {|*args|
     joins(:project).
-    references(:project).
     where(Project.allowed_to_condition(args.shift || User.current, :view_messages, *args))
   }
 
@@ -52,6 +50,11 @@ class Board < ActiveRecord::Base
     name
   end
 
+  # Returns a scope for the board topics (messages without parent)
+  def topics
+    messages.where(:parent_id => nil)
+  end
+
   def valid_parents
     @valid_parents ||= project.boards - self_and_descendants
   end
@@ -63,10 +66,10 @@ class Board < ActiveRecord::Base
   # Updates topics_count, messages_count and last_message_id attributes for +board_id+
   def self.reset_counters!(board_id)
     board_id = board_id.to_i
-    where(["id = ?", board_id]).
-      update_all("topics_count = (SELECT COUNT(*) FROM #{Message.table_name} WHERE board_id=#{board_id} AND parent_id IS NULL)," +
-               " messages_count = (SELECT COUNT(*) FROM #{Message.table_name} WHERE board_id=#{board_id})," +
-               " last_message_id = (SELECT MAX(id) FROM #{Message.table_name} WHERE board_id=#{board_id})")
+    Board.where(:id => board_id).
+      update_all(["topics_count = (SELECT COUNT(*) FROM #{Message.table_name} WHERE board_id=:id AND parent_id IS NULL)," +
+               " messages_count = (SELECT COUNT(*) FROM #{Message.table_name} WHERE board_id=:id)," +
+               " last_message_id = (SELECT MAX(id) FROM #{Message.table_name} WHERE board_id=:id)", :id => board_id])
   end
 
   def self.board_tree(boards, parent_id=nil, level=0)
