@@ -1,5 +1,5 @@
 # Redmine - project management software
-# Copyright (C) 2006-2015  Jean-Philippe Lang
+# Copyright (C) 2006-2016  Jean-Philippe Lang
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -29,7 +29,7 @@ class DocumentCategory < Enumeration
   end
 
   def transfer_relations(to)
-    documents.update_all("category_id = #{to.id}")
+    documents.update_all(:category_id => to.id)
   end
 
   def self.default
