@@ -1,5 +1,5 @@
 # Redmine - project management software
-# Copyright (C) 2006-2014  Jean-Philippe Lang
+# Copyright (C) 2006-2016  Jean-Philippe Lang
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -18,12 +18,12 @@
 class IssueCategory < ActiveRecord::Base
   include Redmine::SafeAttributes
   belongs_to :project
-  belongs_to :assigned_to, :class_name => 'Principal', :foreign_key => 'assigned_to_id'
+  belongs_to :assigned_to, :class_name => 'Principal'
   has_many :issues, :foreign_key => 'category_id', :dependent => :nullify
 
   validates_presence_of :name
   validates_uniqueness_of :name, :scope => [:project_id]
-  validates_length_of :name, :maximum => 30
+  validates_length_of :name, :maximum => 60
   attr_protected :id
 
   safe_attributes 'name', 'assigned_to_id'
