@@ -1,5 +1,5 @@
 # Redmine - project management software
-# Copyright (C) 2006-2014  Jean-Philippe Lang
+# Copyright (C) 2006-2016  Jean-Philippe Lang
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -18,10 +18,11 @@
 class IssueStatus < ActiveRecord::Base
   before_destroy :check_integrity
   has_many :workflows, :class_name => 'WorkflowTransition', :foreign_key => "old_status_id"
+  has_many :workflow_transitions_as_new_status, :class_name => 'WorkflowTransition', :foreign_key => "new_status_id"
   acts_as_list
 
+  after_update :handle_is_closed_change
   before_destroy :delete_workflow_rules
-  after_save     :update_default
 
   validates_presence_of :name
   validates_uniqueness_of :name
@@ -29,18 +30,9 @@ class IssueStatus < ActiveRecord::Base
   validates_inclusion_of :default_done_ratio, :in => 0..100, :allow_nil => true
   attr_protected :id
 
-  scope :sorted, lambda { order("#{table_name}.position ASC") }
+  scope :sorted, lambda { order(:position) }
   scope :named, lambda {|arg| where("LOWER(#{table_name}.name) = LOWER(?)", arg.to_s.strip)}
 
-  def update_default
-    IssueStatus.where(['id <> ?', id]).update_all({:is_default => false}) if self.is_default?
-  end
-
-  # Returns the default status for new issues
-  def self.default
-    where(:is_default => true).first
-  end
-
   # Update all the +Issues+ setting their done_ratio to the value of their +IssueStatus+
   def self.update_issue_done_ratios
     if Issue.use_status_for_done_ratio?
@@ -53,35 +45,28 @@ class IssueStatus < ActiveRecord::Base
   end
 
   # Returns an array of all statuses the given role can switch to
-  # Uses association cache when called more than one time
   def new_statuses_allowed_to(roles, tracker, author=false, assignee=false)
-    if roles && tracker
-      role_ids = roles.collect(&:id)
-      transitions = workflows.select do |w|
-        role_ids.include?(w.role_id) &&
-        w.tracker_id == tracker.id &&
-        ((!w.author && !w.assignee) || (author && w.author) || (assignee && w.assignee))
-      end
-      transitions.map(&:new_status).compact.sort
-    else
-      []
-    end
+    self.class.new_statuses_allowed(self, roles, tracker, author, assignee)
   end
+  alias :find_new_statuses_allowed_to :new_statuses_allowed_to
 
-  # Same thing as above but uses a database query
-  # More efficient than the previous method if called just once
-  def find_new_statuses_allowed_to(roles, tracker, author=false, assignee=false)
+  def self.new_statuses_allowed(status, roles, tracker, author=false, assignee=false)
     if roles.present? && tracker
-      conditions = "(author = :false AND assignee = :false)"
-      conditions << " OR author = :true" if author
-      conditions << " OR assignee = :true" if assignee
+      status_id = status.try(:id) || 0
 
-      workflows.
-        includes(:new_status).
-        where(["role_id IN (:role_ids) AND tracker_id = :tracker_id AND (#{conditions})",
-          {:role_ids => roles.collect(&:id), :tracker_id => tracker.id, :true => true, :false => false}
-          ]).to_a.
-        map(&:new_status).compact.sort
+      scope = IssueStatus.
+        joins(:workflow_transitions_as_new_status).
+        where(:workflows => {:old_status_id => status_id, :role_id => roles.map(&:id), :tracker_id => tracker.id})
+
+      unless author && assignee
+        if author || assignee
+          scope = scope.where("author = ? OR assignee = ?", author, assignee)
+        else
+          scope = scope.where("author = ? AND assignee = ?", false, false)
+        end
+      end
+
+      scope.uniq.to_a.sort
     else
       []
     end
@@ -95,8 +80,30 @@ class IssueStatus < ActiveRecord::Base
 
   private
 
+  # Updates issues closed_on attribute when an existing status is set as closed.
+  def handle_is_closed_change
+    if is_closed_changed? && is_closed == true
+      # First we update issues that have a journal for when the current status was set,
+      # a subselect is used to update all issues with a single query
+      subselect = "SELECT MAX(j.created_on) FROM #{Journal.table_name} j" +
+        " JOIN #{JournalDetail.table_name} d ON d.journal_id = j.id" +
+        " WHERE j.journalized_type = 'Issue' AND j.journalized_id = #{Issue.table_name}.id" +
+        " AND d.property = 'attr' AND d.prop_key = 'status_id' AND d.value = :status_id"
+      Issue.where(:status_id => id, :closed_on => nil).
+        update_all(["closed_on = (#{subselect})", {:status_id => id.to_s}])
+
+      # Then we update issues that don't have a journal which means the
+      # current status was set on creation
+      Issue.where(:status_id => id, :closed_on => nil).update_all("closed_on = created_on")
+    end
+  end
+
   def check_integrity
-    raise "Can't delete status" if Issue.where(:status_id => id).any?
+    if Issue.where(:status_id => id).any?
+      raise "This status is used by some issues"
+    elsif Tracker.where(:default_status_id => id).any?
+      raise "This status is used as the default status by some trackers"
+    end
   end
 
   # Deletes associated workflows
