@@ -1,5 +1,5 @@
 # Redmine - project management software
-# Copyright (C) 2006-2014  Jean-Philippe Lang
+# Copyright (C) 2006-2016  Jean-Philippe Lang
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -18,16 +18,16 @@
 class Message < ActiveRecord::Base
   include Redmine::SafeAttributes
   belongs_to :board
-  belongs_to :author, :class_name => 'User', :foreign_key => 'author_id'
+  belongs_to :author, :class_name => 'User'
   acts_as_tree :counter_cache => :replies_count, :order => "#{Message.table_name}.created_on ASC"
   acts_as_attachable
-  belongs_to :last_reply, :class_name => 'Message', :foreign_key => 'last_reply_id'
+  belongs_to :last_reply, :class_name => 'Message'
   attr_protected :id
 
   acts_as_searchable :columns => ['subject', 'content'],
-                     :scope => preload(:board => :project),
-                     :project_key => "#{Board.table_name}.project_id",
-                     :date_column => "#{table_name}.created_on"
+                     :preload => {:board => :project},
+                     :project_key => "#{Board.table_name}.project_id"
+
   acts_as_event :title => Proc.new {|o| "#{o.board.name}: #{o.subject}"},
                 :description => :content,
                 :group => :parent,
@@ -50,7 +50,6 @@ class Message < ActiveRecord::Base
 
   scope :visible, lambda {|*args|
     joins(:board => :project).
-    references(:board => :project).
     where(Project.allowed_to_condition(args.shift || User.current, :view_messages, *args))
   }
 
@@ -104,6 +103,10 @@ class Message < ActiveRecord::Base
     usr && usr.logged? && (usr.allowed_to?(:delete_messages, project) || (self.author == usr && usr.allowed_to?(:delete_own_messages, project)))
   end
 
+  def notified_users
+    project.notified_users.reject {|user| !visible?(user)}
+  end
+
   private
 
   def add_author_as_watcher
