@@ -1,5 +1,5 @@
 # Redmine - project management software
-# Copyright (C) 2006-2014  Jean-Philippe Lang
+# Copyright (C) 2006-2016  Jean-Philippe Lang
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -18,7 +18,7 @@
 class News < ActiveRecord::Base
   include Redmine::SafeAttributes
   belongs_to :project
-  belongs_to :author, :class_name => 'User', :foreign_key => 'author_id'
+  belongs_to :author, :class_name => 'User'
   has_many :comments, lambda {order("created_on")}, :as => :commented, :dependent => :delete_all
 
   validates_presence_of :title, :description
@@ -26,9 +26,10 @@ class News < ActiveRecord::Base
   validates_length_of :summary, :maximum => 255
   attr_protected :id
 
-  acts_as_attachable :delete_permission => :manage_news
+  acts_as_attachable :edit_permission => :manage_news,
+                     :delete_permission => :manage_news
   acts_as_searchable :columns => ['title', 'summary', "#{table_name}.description"],
-                     :scope => preload(:project)
+                     :preload => :project
   acts_as_event :url => Proc.new {|o| {:controller => 'news', :action => 'show', :id => o.id}}
   acts_as_activity_provider :scope => preload(:project, :author),
                             :author_key => :author_id
@@ -39,7 +40,6 @@ class News < ActiveRecord::Base
 
   scope :visible, lambda {|*args|
     joins(:project).
-    references([:author, :project]).
     where(Project.allowed_to_condition(args.shift || User.current, :view_news, *args))
   }
 
@@ -54,25 +54,34 @@ class News < ActiveRecord::Base
     user.allowed_to?(:comment_news, project)
   end
 
+  def notified_users
+    project.users.select {|user| user.notify_about?(self) && user.allowed_to?(:view_news, project)}
+  end
+
   def recipients
-    project.users.select {|user| user.notify_about?(self) && user.allowed_to?(:view_news, project)}.map(&:mail)
+    notified_users.map(&:mail)
+  end
+
+  # Returns the users that should be cc'd when a new news is added
+  def notified_watchers_for_added_news
+    watchers = []
+    if m = project.enabled_module('news')
+      watchers = m.notified_watchers
+      unless project.is_public?
+        watchers = watchers.select {|user| project.users.include?(user)}
+      end
+    end
+    watchers
   end
 
   # Returns the email addresses that should be cc'd when a new news is added
   def cc_for_added_news
-    cc = []
-    if m = project.enabled_module('news')
-      cc = m.notified_watchers
-      unless project.is_public?
-        cc = cc.select {|user| project.users.include?(user)}
-      end
-    end
-    cc.map(&:mail)
+    notified_watchers_for_added_news.map(&:mail)
   end
 
   # returns latest news for projects visible by user
   def self.latest(user = User.current, count = 5)
-    visible(user).joins([:author, :project]).order("#{News.table_name}.created_on DESC").limit(count).to_a
+    visible(user).preload(:author, :project).order("#{News.table_name}.created_on DESC").limit(count).to_a
   end
 
   private
