@@ -1,5 +1,5 @@
 # Redmine - project management software
-# Copyright (C) 2006-2014  Jean-Philippe Lang
+# Copyright (C) 2006-2016  Jean-Philippe Lang
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -28,8 +28,7 @@ class Principal < ActiveRecord::Base
   has_many :memberships,
            lambda {preload(:project, :roles).
                    joins(:project).
-                   where("#{Project.table_name}.status<>#{Project::STATUS_ARCHIVED}").
-                   order("#{Project.table_name}.name")},
+                   where("#{Project.table_name}.status<>#{Project::STATUS_ARCHIVED}")},
            :class_name => 'Member',
            :foreign_key => 'user_id'
   has_many :projects, :through => :memberships
@@ -38,13 +37,38 @@ class Principal < ActiveRecord::Base
   # Groups and active users
   scope :active, lambda { where(:status => STATUS_ACTIVE) }
 
+  scope :visible, lambda {|*args|
+    user = args.first || User.current
+
+    if user.admin?
+      all
+    else
+      view_all_active = false
+      if user.memberships.to_a.any?
+        view_all_active = user.memberships.any? {|m| m.roles.any? {|r| r.users_visibility == 'all'}}
+      else
+        view_all_active = user.builtin_role.users_visibility == 'all'
+      end
+
+      if view_all_active
+        active
+      else
+        # self and members of visible projects
+        active.where("#{table_name}.id = ? OR #{table_name}.id IN (SELECT user_id FROM #{Member.table_name} WHERE project_id IN (?))",
+          user.id, user.visible_project_ids
+        )
+      end
+    end
+  }
+
   scope :like, lambda {|q|
     q = q.to_s
     if q.blank?
       where({})
     else
       pattern = "%#{q}%"
-      sql = %w(login firstname lastname mail).map {|column| "LOWER(#{table_name}.#{column}) LIKE LOWER(:p)"}.join(" OR ")
+      sql = %w(login firstname lastname).map {|column| "LOWER(#{table_name}.#{column}) LIKE LOWER(:p)"}.join(" OR ")
+      sql << " OR #{table_name}.id IN (SELECT user_id FROM #{EmailAddress.table_name} WHERE LOWER(address) LIKE LOWER(:p))"
       params = {:p => pattern}
       if q =~ /^(.+)\s+(.+)$/
         a, b = "#{$1}%", "#{$2}%"
@@ -84,11 +108,28 @@ class Principal < ActiveRecord::Base
     to_s
   end
 
+  def mail=(*args)
+    nil
+  end
+
+  def mail
+    nil
+  end
+
+  def visible?(user=User.current)
+    Principal.visible(user).where(:id => id).first == self
+  end
+
+  # Return true if the principal is a member of project
+  def member_of?(project)
+    projects.to_a.include?(project)
+  end
+
   def <=>(principal)
     if principal.nil?
       -1
     elsif self.class.name == principal.class.name
-      self.to_s.downcase <=> principal.to_s.downcase
+      self.to_s.casecmp(principal.to_s)
     else
       # groups after users
       principal.class.name <=> self.class.name
@@ -104,6 +145,29 @@ class Principal < ActiveRecord::Base
     columns.uniq.map {|field| "#{table}.#{field}"}
   end
 
+  # Returns the principal that matches the keyword among principals
+  def self.detect_by_keyword(principals, keyword)
+    keyword = keyword.to_s
+    return nil if keyword.blank?
+
+    principal = nil
+    principal ||= principals.detect {|a| keyword.casecmp(a.login.to_s) == 0}
+    principal ||= principals.detect {|a| keyword.casecmp(a.mail.to_s) == 0}
+
+    if principal.nil? && keyword.match(/ /)
+      firstname, lastname = *(keyword.split) # "First Last Throwaway"
+      principal ||= principals.detect {|a|
+                                 a.is_a?(User) &&
+                                   firstname.casecmp(a.firstname.to_s) == 0 &&
+                                   lastname.casecmp(a.lastname.to_s) == 0
+                               }
+    end
+    if principal.nil?
+      principal ||= principals.detect {|a| keyword.casecmp(a.name) == 0}
+    end
+    principal
+  end
+
   protected
 
   # Make sure we don't try to insert NULL values (see #4632)
@@ -112,7 +176,9 @@ class Principal < ActiveRecord::Base
     self.hashed_password ||= ''
     self.firstname ||= ''
     self.lastname ||= ''
-    self.mail ||= ''
     true
   end
 end
+
+require_dependency "user"
+require_dependency "group"
