@@ -1,5 +1,5 @@
 # Redmine - project management software
-# Copyright (C) 2006-2014  Jean-Philippe Lang
+# Copyright (C) 2006-2016  Jean-Philippe Lang
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -17,6 +17,7 @@
 
 class Project < ActiveRecord::Base
   include Redmine::SafeAttributes
+  include Redmine::NestedSet::ProjectNestedSet
 
   # Project statuses
   STATUS_ACTIVE     = 1
@@ -28,21 +29,16 @@ class Project < ActiveRecord::Base
 
   # Specific overridden Activities
   has_many :time_entry_activities
+  has_many :memberships, :class_name => 'Member', :inverse_of => :project
+  # Memberships of active users only
   has_many :members,
-           lambda { joins(:principal, :roles).
-                    references(:principal, :roles).
-                    where("#{Principal.table_name}.type='User' AND #{Principal.table_name}.status=#{Principal::STATUS_ACTIVE}") }
-  has_many :memberships, :class_name => 'Member'
-  has_many :member_principals,
-           lambda { joins(:principal).
-                    references(:principal).
-                    where("#{Principal.table_name}.type='Group' OR (#{Principal.table_name}.type='User' AND #{Principal.table_name}.status=#{Principal::STATUS_ACTIVE})")},
-    :class_name => 'Member'
+           lambda { joins(:principal).where(:users => {:type => 'User', :status => Principal::STATUS_ACTIVE}) }
   has_many :enabled_modules, :dependent => :delete_all
-  has_and_belongs_to_many :trackers, lambda {order("#{Tracker.table_name}.position")}
+  has_and_belongs_to_many :trackers, lambda {order(:position)}
   has_many :issues, :dependent => :destroy
   has_many :issue_changes, :through => :issues, :source => :journals
   has_many :versions, lambda {order("#{Version.table_name}.effective_date DESC, #{Version.table_name}.name DESC")}, :dependent => :destroy
+  belongs_to :default_version, :class_name => 'Version'
   has_many :time_entries, :dependent => :destroy
   has_many :queries, :class_name => 'IssueQuery', :dependent => :delete_all
   has_many :documents, :dependent => :destroy
@@ -60,12 +56,12 @@ class Project < ActiveRecord::Base
                           :join_table => "#{table_name_prefix}custom_fields_projects#{table_name_suffix}",
                           :association_foreign_key => 'custom_field_id'
 
-  acts_as_nested_set :dependent => :destroy
   acts_as_attachable :view_permission => :view_files,
+                     :edit_permission => :manage_files,
                      :delete_permission => :manage_files
 
   acts_as_customizable
-  acts_as_searchable :columns => ['name', 'identifier', 'description'], :project_key => 'id', :permission => nil
+  acts_as_searchable :columns => ['name', 'identifier', 'description'], :project_key => "#{Project.table_name}.id", :permission => nil
   acts_as_event :title => Proc.new {|o| "#{l(:label_project)}: #{o.name}"},
                 :url => Proc.new {|o| {:controller => 'projects', :action => 'show', :id => o}},
                 :author => nil
@@ -73,8 +69,7 @@ class Project < ActiveRecord::Base
   attr_protected :status
 
   validates_presence_of :name, :identifier
-  validates_uniqueness_of :identifier
-  validates_associated :repository, :wiki
+  validates_uniqueness_of :identifier, :if => Proc.new {|p| p.identifier_changed?}
   validates_length_of :name, :maximum => 255
   validates_length_of :homepage, :maximum => 255
   validates_length_of :identifier, :in => 1..IDENTIFIER_MAX_LENGTH
@@ -82,9 +77,11 @@ class Project < ActiveRecord::Base
   validates_format_of :identifier, :with => /\A(?!\d+$)[a-z0-9\-_]*\z/, :if => Proc.new { |p| p.identifier_changed? }
   # reserved words
   validates_exclusion_of :identifier, :in => %w( new )
+  validate :validate_parent
 
-  after_save :update_position_under_parent, :if => Proc.new {|project| project.name_changed?}
   after_save :update_inherited_members, :if => Proc.new {|project| project.inherit_members_changed?}
+  after_save :remove_inherited_member_roles, :add_inherited_member_roles, :if => Proc.new {|project| project.parent_id_changed?}
+  after_update :update_versions_from_hierarchy_change, :if => Proc.new {|project| project.parent_id_changed?}
   before_destroy :delete_all_members
 
   scope :has_module, lambda {|mod|
@@ -113,6 +110,10 @@ class Project < ActiveRecord::Base
       where("LOWER(identifier) LIKE :p OR LOWER(name) LIKE :p", :p => pattern)
     end
   }
+  scope :sorted, lambda {order(:lft)}
+  scope :having_trackers, lambda {
+    where("#{Project.table_name}.id IN (SELECT DISTINCT project_id FROM #{table_name_prefix}projects_trackers#{table_name_suffix})")
+  }
 
   def initialize(attributes=nil, *args)
     super
@@ -148,7 +149,10 @@ class Project < ActiveRecord::Base
   # returns latest created projects
   # non public projects will be returned only if user is a member of those
   def self.latest(user=nil, count=5)
-    visible(user).limit(count).order("created_on DESC").to_a
+    visible(user).limit(count).
+      order(:created_on => :desc).
+      where("#{table_name}.created_on >= ?", 30.days.ago).
+      to_a
   end
 
   # Returns true if the project is visible to +user+ or to the current user.
@@ -179,9 +183,8 @@ class Project < ActiveRecord::Base
       # If the permission belongs to a project module, make sure the module is enabled
       base_statement << " AND #{Project.table_name}.id IN (SELECT em.project_id FROM #{EnabledModule.table_name} em WHERE em.name='#{perm.project_module}')"
     end
-    if options[:project]
-      project_statement = "#{Project.table_name}.id = #{options[:project].id}"
-      project_statement << " OR (#{Project.table_name}.lft > #{options[:project].lft} AND #{Project.table_name}.rgt < #{options[:project].rgt})" if options[:with_subprojects]
+    if project = options[:project]
+      project_statement = project.project_condition(options[:with_subprojects])
       base_statement = "(#{project_statement}) AND (#{base_statement})"
     end
 
@@ -192,14 +195,16 @@ class Project < ActiveRecord::Base
       unless options[:member]
         role = user.builtin_role
         if role.allowed_to?(permission)
-          statement_by_role[role] = "#{Project.table_name}.is_public = #{connection.quoted_true}"
+          s = "#{Project.table_name}.is_public = #{connection.quoted_true}"
+          if user.id
+            s = "(#{s} AND #{Project.table_name}.id NOT IN (SELECT project_id FROM #{Member.table_name} WHERE user_id = #{user.id}))"
+          end
+          statement_by_role[role] = s
         end
       end
-      if user.logged?
-        user.projects_by_role.each do |role, projects|
-          if role.allowed_to?(permission) && projects.any?
-            statement_by_role[role] = "#{Project.table_name}.id IN (#{projects.collect(&:id).join(',')})"
-          end
+      user.projects_by_role.each do |role, projects|
+        if role.allowed_to?(permission) && projects.any?
+          statement_by_role[role] = "#{Project.table_name}.id IN (#{projects.collect(&:id).join(',')})"
         end
       end
       if statement_by_role.empty?
@@ -217,6 +222,16 @@ class Project < ActiveRecord::Base
     end
   end
 
+  def override_roles(role)
+    @override_members ||= memberships.
+      joins(:principal).
+      where(:users => {:type => ['GroupAnonymous', 'GroupNonMember']}).to_a
+
+    group_class = role.anonymous? ? GroupAnonymous : GroupNonMember
+    member = @override_members.detect {|m| m.principal.is_a? group_class}
+    member ? member.roles.to_a : [role]
+  end
+
   def principals
     @principals ||= Principal.active.joins(:members).where("#{Member.table_name}.project_id = ?", id).uniq
   end
@@ -227,11 +242,17 @@ class Project < ActiveRecord::Base
 
   # Returns the Systemwide and project specific activities
   def activities(include_inactive=false)
-    if include_inactive
-      return all_activities
-    else
-      return active_activities
+    t = TimeEntryActivity.table_name
+    scope = TimeEntryActivity.where("#{t}.project_id IS NULL OR #{t}.project_id = ?", id)
+
+    overridden_activity_ids = self.time_entry_activities.pluck(:parent_id).compact
+    if overridden_activity_ids.any?
+      scope = scope.where("#{t}.id NOT IN (?)", overridden_activity_ids)
     end
+    unless include_inactive
+      scope = scope.active
+    end
+    scope
   end
 
   # Will create a new Project specific Activity or update an existing one
@@ -262,8 +283,8 @@ class Project < ActiveRecord::Base
           raise ActiveRecord::Rollback, "Overriding TimeEntryActivity was not successfully saved"
         else
           self.time_entries.
-            where(["activity_id = ?", parent_activity.id]).
-            update_all("activity_id = #{project_activity.id}")
+            where(:activity_id => parent_activity.id).
+            update_all(:activity_id => project_activity.id)
         end
       end
     end
@@ -309,12 +330,18 @@ class Project < ActiveRecord::Base
     @actions_allowed = nil
     @start_date = nil
     @due_date = nil
+    @override_members = nil
+    @assignable_users = nil
     base_reload(*args)
   end
 
   def to_param
-    # id is used for projects with a numeric identifier (compatibility)
-    @to_param ||= (identifier.to_s =~ %r{^\d*$} ? id.to_s : identifier)
+    if new_record?
+      nil
+    else
+      # id is used for projects with a numeric identifier (compatibility)
+      @to_param ||= (identifier.to_s =~ %r{^\d*$} ? id.to_s : identifier)
+    end
   end
 
   def active?
@@ -329,12 +356,13 @@ class Project < ActiveRecord::Base
   def archive
     # Check that there is no issue of a non descendant project that is assigned
     # to one of the project or descendant versions
-    v_ids = self_and_descendants.collect {|p| p.version_ids}.flatten
-    if v_ids.any? &&
+    version_ids = self_and_descendants.joins(:versions).pluck("#{Version.table_name}.id")
+
+    if version_ids.any? &&
       Issue.
         includes(:project).
         where("#{Project.table_name}.lft < ? OR #{Project.table_name}.rgt > ?", lft, rgt).
-        where("#{Issue.table_name}.fixed_version_id IN (?)", v_ids).
+        where(:fixed_version_id => version_ids).
         exists?
       return false
     end
@@ -361,11 +389,11 @@ class Project < ActiveRecord::Base
 
   # Returns an array of projects the project can be moved to
   # by the current user
-  def allowed_parents
+  def allowed_parents(user=User.current)
     return @allowed_parents if @allowed_parents
-    @allowed_parents = Project.where(Project.allowed_to_condition(User.current, :add_subprojects)).to_a
+    @allowed_parents = Project.allowed_to(user, :add_subprojects).to_a
     @allowed_parents = @allowed_parents - self_and_descendants
-    if User.current.allowed_to?(:add_project, nil, :global => true) || (!new_record? && parent.nil?)
+    if user.allowed_to?(:add_project, nil, :global => true) || (!new_record? && parent.nil?)
       @allowed_parents << nil
     end
     unless parent.nil? || @allowed_parents.empty? || @allowed_parents.include?(parent)
@@ -376,57 +404,21 @@ class Project < ActiveRecord::Base
 
   # Sets the parent of the project with authorization check
   def set_allowed_parent!(p)
-    unless p.nil? || p.is_a?(Project)
-      if p.to_s.blank?
-        p = nil
-      else
-        p = Project.find_by_id(p)
-        return false unless p
-      end
-    end
-    if p.nil?
-      if !new_record? && allowed_parents.empty?
-        return false
-      end
-    elsif !allowed_parents.include?(p)
-      return false
-    end
-    set_parent!(p)
+    ActiveSupport::Deprecation.warn "Project#set_allowed_parent! is deprecated and will be removed in Redmine 4, use #safe_attributes= instead."
+    p = p.id if p.is_a?(Project)
+    send :safe_attributes, {:project_id => p}
+    save
   end
 
-  # Sets the parent of the project
+  # Sets the parent of the project and saves the project
   # Argument can be either a Project, a String, a Fixnum or nil
   def set_parent!(p)
-    unless p.nil? || p.is_a?(Project)
-      if p.to_s.blank?
-        p = nil
-      else
-        p = Project.find_by_id(p)
-        return false unless p
-      end
-    end
-    if p == parent && !p.nil?
-      # Nothing to do
-      true
-    elsif p.nil? || (p.active? && move_possible?(p))
-      set_or_update_position_under(p)
-      Issue.update_versions_from_hierarchy_change(self)
-      true
+    if p.is_a?(Project)
+      self.parent = p
     else
-      # Can not move to the given target
-      false
-    end
-  end
-
-  # Recalculates all lft and rgt values based on project names
-  # Unlike Project.rebuild!, these values are recalculated even if the tree "looks" valid
-  # Used in BuildProjectsTree migration
-  def self.rebuild_tree!
-    transaction do
-      update_all "lft = NULL, rgt = NULL"
-      rebuild!(false)
-      all.each { |p| p.set_or_update_position_under(p.parent) }
+      self.parent_id = p
     end
+    save
   end
 
   # Returns an array of the trackers used by the project and its active sub projects
@@ -434,10 +426,9 @@ class Project < ActiveRecord::Base
     @rolled_up_trackers ||=
       Tracker.
         joins(:projects).
-        references(:project).
         joins("JOIN #{EnabledModule.table_name} ON #{EnabledModule.table_name}.project_id = #{Project.table_name}.id AND #{EnabledModule.table_name}.name = 'issue_tracking'").
-        select("DISTINCT #{Tracker.table_name}.*").
-        where("#{Project.table_name}.lft >= ? AND #{Project.table_name}.rgt <= ? AND #{Project.table_name}.status <> #{STATUS_ARCHIVED}", lft, rgt).
+        where("#{Project.table_name}.lft >= ? AND #{Project.table_name}.rgt <= ? AND #{Project.table_name}.status <> ?", lft, rgt, STATUS_ARCHIVED).
+        uniq.
         sorted.
         to_a
   end
@@ -458,7 +449,6 @@ class Project < ActiveRecord::Base
     @rolled_up_versions ||=
       Version.
         joins(:project).
-        references(:project).
         where("#{Project.table_name}.lft >= ? AND #{Project.table_name}.rgt <= ? AND #{Project.table_name}.status <> ?", lft, rgt, STATUS_ARCHIVED)
   end
 
@@ -467,7 +457,6 @@ class Project < ActiveRecord::Base
     if new_record?
       Version.
         joins(:project).
-        references(:project).
         preload(:project).
         where("#{Project.table_name}.status <> ? AND #{Version.table_name}.sharing = 'system'", STATUS_ARCHIVED)
     else
@@ -475,7 +464,6 @@ class Project < ActiveRecord::Base
         r = root? ? self : root
         Version.
           joins(:project).
-          references(:project).
           preload(:project).
           where("#{Project.table_name}.id = #{id}" +
                   " OR (#{Project.table_name}.status <> #{Project::STATUS_ARCHIVED} AND (" +
@@ -499,6 +487,15 @@ class Project < ActiveRecord::Base
     end
   end
 
+  # Adds user as a project member with the default role
+  # Used for when a non-admin user creates a project
+  def add_default_member(user)
+    role = Role.givable.find_by_id(Setting.new_project_user_role_id.to_i) || Role.givable.first
+    member = Member.new(:project => self, :principal => user, :roles => [role])
+    self.members << member
+    member
+  end
+
   # Deletes all project's members
   def delete_all_members
     me, mr = Member.table_name, MemberRole.table_name
@@ -506,10 +503,17 @@ class Project < ActiveRecord::Base
     Member.delete_all(['project_id = ?', id])
   end
 
-  # Users/groups issues can be assigned to
+  # Return a Principal scope of users/groups issues can be assigned to
   def assignable_users
-    assignable = Setting.issue_group_assignment? ? member_principals : members
-    assignable.select {|m| m.roles.detect {|role| role.assignable?}}.collect {|m| m.principal}.sort
+    types = ['User']
+    types << 'Group' if Setting.issue_group_assignment?
+
+    @assignable_users ||= Principal.
+      active.
+      joins(:members => :roles).
+      where(:type => types, :members => {:project_id => id}, :roles => {:assignable => true}).
+      uniq.
+      sorted
   end
 
   # Returns the mail addresses of users that should be always notified on project events
@@ -526,17 +530,17 @@ class Project < ActiveRecord::Base
   # Returns a scope of all custom fields enabled for project issues
   # (explicitly associated custom fields and custom fields enabled for all projects)
   def all_issue_custom_fields
-    @all_issue_custom_fields ||= IssueCustomField.
-      sorted.
-      where("is_for_all = ? OR id IN (SELECT DISTINCT cfp.custom_field_id" +
-        " FROM #{table_name_prefix}custom_fields_projects#{table_name_suffix} cfp" +
-        " WHERE cfp.project_id = ?)", true, id)
-  end
-
-  # Returns an array of all custom fields enabled for project time entries
-  # (explictly associated custom fields and custom fields enabled for all projects)
-  def all_time_entry_custom_fields
-    @all_time_entry_custom_fields ||= (TimeEntryCustomField.for_all + time_entry_custom_fields).uniq.sort
+    if new_record?
+      @all_issue_custom_fields ||= IssueCustomField.
+        sorted.
+        where("is_for_all = ? OR id IN (?)", true, issue_custom_field_ids)
+    else
+      @all_issue_custom_fields ||= IssueCustomField.
+        sorted.
+        where("is_for_all = ? OR id IN (SELECT DISTINCT cfp.custom_field_id" +
+          " FROM #{table_name_prefix}custom_fields_projects#{table_name_suffix} cfp" +
+          " WHERE cfp.project_id = ?)", true, id)
+    end
   end
 
   def project
@@ -544,7 +548,7 @@ class Project < ActiveRecord::Base
   end
 
   def <=>(project)
-    name.downcase <=> project.name.downcase
+    name.casecmp(project.name)
   end
 
   def to_s
@@ -686,7 +690,9 @@ class Project < ActiveRecord::Base
     'custom_field_values',
     'custom_fields',
     'tracker_ids',
-    'issue_custom_field_ids'
+    'issue_custom_field_ids',
+    'parent_id',
+    'default_version_id'
 
   safe_attributes 'enabled_module_names',
     :if => lambda {|project, user| project.new_record? || user.allowed_to?(:select_project_modules, project) }
@@ -694,13 +700,23 @@ class Project < ActiveRecord::Base
   safe_attributes 'inherit_members',
     :if => lambda {|project, user| project.parent.nil? || project.parent.visible?(user)}
 
-  # Returns an array of projects that are in this project's hierarchy
-  #
-  # Example: parents, children, siblings
-  def hierarchy
-    parents = project.self_and_ancestors || []
-    descendants = project.descendants || []
-    project_hierarchy = parents | descendants # Set union
+  def safe_attributes=(attrs, user=User.current)
+    return unless attrs.is_a?(Hash)
+    attrs = attrs.deep_dup
+
+    @unallowed_parent_id = nil
+    if new_record? || attrs.key?('parent_id')
+      parent_id_param = attrs['parent_id'].to_s
+      if new_record? || parent_id_param != parent_id.to_s
+        p = parent_id_param.present? ? Project.find_by_id(parent_id_param) : nil
+        unless allowed_parents(user).include?(p)
+          attrs.delete('parent_id')
+          @unallowed_parent_id = true
+        end
+      end
+    end
+
+    super(attrs, user)
   end
 
   # Returns an auto-generated project identifier based on the last identifier used
@@ -738,9 +754,15 @@ class Project < ActiveRecord::Base
         end
         Redmine::Hook.call_hook(:model_project_copy_before_save, :source_project => project, :destination_project => self)
         save
+      else
+        false
       end
     end
-    true
+  end
+
+  def member_principals
+    ActiveSupport::Deprecation.warn "Project#member_principals is deprecated and will be removed in Redmine 4.0. Use #memberships.active instead."
+    memberships.active
   end
 
   # Returns a new unsaved Project instance with attributes copied from +project+
@@ -749,7 +771,7 @@ class Project < ActiveRecord::Base
     # clear unique attributes
     attributes = project.attributes.dup.except('id', 'name', 'identifier', 'status', 'parent_id', 'lft', 'rgt')
     copy = Project.new(attributes)
-    copy.enabled_modules = project.enabled_modules
+    copy.enabled_module_names = project.enabled_module_names
     copy.trackers = project.trackers
     copy.custom_values = project.custom_values.collect {|v| v.clone}
     copy.issue_custom_fields = project.issue_custom_fields
@@ -770,11 +792,6 @@ class Project < ActiveRecord::Base
 
   private
 
-  def after_parent_changed(parent_was)
-    remove_inherited_member_roles
-    add_inherited_member_roles
-  end
-
   def update_inherited_members
     if parent
       if inherit_members? && !inherit_members_was
@@ -805,6 +822,21 @@ class Project < ActiveRecord::Base
         end
         member.save!
       end
+      memberships.reset
+    end
+  end
+
+  def update_versions_from_hierarchy_change
+    Issue.update_versions_from_hierarchy_change(self)
+  end
+
+  def validate_parent
+    if @unallowed_parent_id
+      errors.add(:parent_id, :invalid)
+    elsif parent_id_changed?
+      unless parent.nil? || (parent.active? && move_possible?(parent))
+        errors.add(:parent_id, :invalid)
+      end
     end
   end
 
@@ -880,6 +912,22 @@ class Project < ActiveRecord::Base
       if issue.fixed_version && issue.fixed_version.project == project
         new_issue.fixed_version = self.versions.detect {|v| v.name == issue.fixed_version.name}
       end
+      # Reassign version custom field values
+      new_issue.custom_field_values.each do |custom_value|
+        if custom_value.custom_field.field_format == 'version' && custom_value.value.present?
+          versions = Version.where(:id => custom_value.value).to_a
+          new_value = versions.map do |version|
+            if version.project == project
+              self.versions.detect {|v| v.name == version.name}.try(:id)
+            else
+              version.id
+            end
+          end
+          new_value.compact!
+          new_value = new_value.first unless custom_value.custom_field.multiple?
+          custom_value.value = new_value
+        end
+      end
       # Reassign the category by name, since names are unique per project
       if issue.category
         new_issue.category = self.issue_categories.detect {|c| c.name == issue.category.name}
@@ -893,7 +941,7 @@ class Project < ActiveRecord::Base
 
       self.issues << new_issue
       if new_issue.new_record?
-        logger.info "Project#copy_issues: issue ##{issue.id} could not be copied: #{new_issue.errors.full_messages}" if logger && logger.info
+        logger.info "Project#copy_issues: issue ##{issue.id} could not be copied: #{new_issue.errors.full_messages}" if logger && logger.info?
       else
         issues_map[issue.id] = new_issue unless new_issue.new_record?
       end
@@ -959,10 +1007,11 @@ class Project < ActiveRecord::Base
   def copy_queries(project)
     project.queries.each do |query|
       new_query = IssueQuery.new
-      new_query.attributes = query.attributes.dup.except("id", "project_id", "sort_criteria")
+      new_query.attributes = query.attributes.dup.except("id", "project_id", "sort_criteria", "user_id", "type")
       new_query.sort_criteria = query.sort_criteria if query.sort_criteria
       new_query.project = self
       new_query.user_id = query.user_id
+      new_query.role_ids = query.role_ids if query.visibility == IssueQuery::VISIBILITY_ROLES
       self.queries << new_query
     end
   end
@@ -988,42 +1037,6 @@ class Project < ActiveRecord::Base
     @actions_allowed ||= allowed_permissions.inject([]) { |actions, permission| actions += Redmine::AccessControl.allowed_actions(permission) }.flatten
   end
 
-  # Returns all the active Systemwide and project specific activities
-  def active_activities
-    overridden_activity_ids = self.time_entry_activities.collect(&:parent_id)
-
-    if overridden_activity_ids.empty?
-      return TimeEntryActivity.shared.active
-    else
-      return system_activities_and_project_overrides
-    end
-  end
-
-  # Returns all the Systemwide and project specific activities
-  # (inactive and active)
-  def all_activities
-    overridden_activity_ids = self.time_entry_activities.collect(&:parent_id)
-
-    if overridden_activity_ids.empty?
-      return TimeEntryActivity.shared
-    else
-      return system_activities_and_project_overrides(true)
-    end
-  end
-
-  # Returns the systemwide active activities merged with the project specific overrides
-  def system_activities_and_project_overrides(include_inactive=false)
-    t = TimeEntryActivity.table_name
-    scope = TimeEntryActivity.where(
-      "(#{t}.project_id IS NULL AND #{t}.id NOT IN (?)) OR (#{t}.project_id = ?)",
-      time_entry_activities.map(&:parent_id), id
-    )
-    unless include_inactive
-      scope = scope.active
-    end
-    scope
-  end
-
   # Archives subprojects recursively
   def archive!
     children.each do |subproject|
@@ -1031,34 +1044,4 @@ class Project < ActiveRecord::Base
     end
     update_attribute :status, STATUS_ARCHIVED
   end
-
-  def update_position_under_parent
-    set_or_update_position_under(parent)
-  end
-
-  public
-
-  # Inserts/moves the project so that target's children or root projects stay alphabetically sorted
-  def set_or_update_position_under(target_parent)
-    parent_was = parent
-    sibs = (target_parent.nil? ? self.class.roots : target_parent.children)
-    to_be_inserted_before = sibs.sort_by {|c| c.name.to_s.downcase}.detect {|c| c.name.to_s.downcase > name.to_s.downcase }
-
-    if to_be_inserted_before
-      move_to_left_of(to_be_inserted_before)
-    elsif target_parent.nil?
-      if sibs.empty?
-        # move_to_root adds the project in first (ie. left) position
-        move_to_root
-      else
-        move_to_right_of(sibs.last) unless self == sibs.last
-      end
-    else
-      # move_to_child_of adds the project in last (ie.right) position
-      move_to_child_of(target_parent)
-    end
-    if parent_was != target_parent
-      after_parent_changed(parent_was)
-    end
-  end
 end
