@@ -1,5 +1,5 @@
 # Redmine - project management software
-# Copyright (C) 2006-2014  Jean-Philippe Lang
+# Copyright (C) 2006-2016  Jean-Philippe Lang
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -24,6 +24,7 @@ class Tracker < ActiveRecord::Base
   CORE_FIELDS_ALL = (CORE_FIELDS_UNDISABLABLE + CORE_FIELDS).freeze
 
   before_destroy :check_integrity
+  belongs_to :default_status, :class_name => 'IssueStatus'
   has_many :issues
   has_many :workflow_rules, :dependent => :delete_all do
     def copy(source_tracker)
@@ -37,11 +38,12 @@ class Tracker < ActiveRecord::Base
 
   attr_protected :fields_bits
 
+  validates_presence_of :default_status
   validates_presence_of :name
   validates_uniqueness_of :name
   validates_length_of :name, :maximum => 30
 
-  scope :sorted, lambda { order("#{table_name}.position ASC") }
+  scope :sorted, lambda { order(:position) }
   scope :named, lambda {|arg| where("LOWER(#{table_name}.name) = LOWER(?)", arg.to_s.strip)}
 
   def to_s; name end
@@ -53,17 +55,15 @@ class Tracker < ActiveRecord::Base
   # Returns an array of IssueStatus that are used
   # in the tracker's workflows
   def issue_statuses
-    if @issue_statuses
-      return @issue_statuses
-    elsif new_record?
-      return []
-    end
+    @issue_statuses ||= IssueStatus.where(:id => issue_status_ids).to_a.sort
+  end
 
-    ids = WorkflowTransition.
-            connection.select_rows("SELECT DISTINCT old_status_id, new_status_id FROM #{WorkflowTransition.table_name} WHERE tracker_id = #{id} AND type = 'WorkflowTransition'").
-            flatten.
-            uniq
-    @issue_statuses = IssueStatus.where(:id => ids).to_a.sort
+  def issue_status_ids
+    if new_record?
+      []
+    else
+      @issue_status_ids ||= WorkflowTransition.where(:tracker_id => id).uniq.pluck(:old_status_id, :new_status_id).flatten.uniq
+    end
   end
 
   def disabled_core_fields
@@ -109,6 +109,6 @@ class Tracker < ActiveRecord::Base
 
 private
   def check_integrity
-    raise Exception.new("Can't delete tracker") if Issue.where(:tracker_id => self.id).any?
+    raise Exception.new("Cannot delete tracker") if Issue.where(:tracker_id => self.id).any?
   end
 end
