@@ -1,5 +1,5 @@
 # Redmine - project management software
-# Copyright (C) 2006-2014  Jean-Philippe Lang
+# Copyright (C) 2006-2016  Jean-Philippe Lang
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -19,16 +19,18 @@ require 'zlib'
 
 class WikiContent < ActiveRecord::Base
   self.locking_column = 'version'
-  belongs_to :page, :class_name => 'WikiPage', :foreign_key => 'page_id'
-  belongs_to :author, :class_name => 'User', :foreign_key => 'author_id'
+  belongs_to :page, :class_name => 'WikiPage'
+  belongs_to :author, :class_name => 'User'
   validates_presence_of :text
-  validates_length_of :comments, :maximum => 255, :allow_nil => true
+  validates_length_of :comments, :maximum => 1024, :allow_nil => true
   attr_protected :id
 
   acts_as_versioned
 
   after_save :send_notification
 
+  scope :without_text, lambda {select(:id, :page_id, :version, :updated_on)}
+
   def visible?(user=User.current)
     page.visible?(user)
   end
@@ -41,11 +43,13 @@ class WikiContent < ActiveRecord::Base
     page.nil? ? [] : page.attachments
   end
 
+  def notified_users
+    project.notified_users.reject {|user| !visible?(user)}
+  end
+
   # Returns the mail addresses of users that should be notified
   def recipients
-    notified = project.notified_users
-    notified.reject! {|user| !visible?(user)}
-    notified.collect(&:mail)
+    notified_users.collect(&:mail)
   end
 
   # Return true if the content is the current page content
@@ -54,8 +58,8 @@ class WikiContent < ActiveRecord::Base
   end
 
   class Version
-    belongs_to :page, :class_name => '::WikiPage', :foreign_key => 'page_id'
-    belongs_to :author, :class_name => '::User', :foreign_key => 'author_id'
+    belongs_to :page, :class_name => '::WikiPage'
+    belongs_to :author, :class_name => '::User'
     attr_protected :data
 
     acts_as_event :title => Proc.new {|o| "#{l(:label_wiki_edit)}: #{o.page.title} (##{o.version})"},
