@@ -1,5 +1,5 @@
 # Redmine - project management software
-# Copyright (C) 2006-2014  Jean-Philippe Lang
+# Copyright (C) 2006-2016  Jean-Philippe Lang
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -18,7 +18,22 @@
 class WikiRedirect < ActiveRecord::Base
   belongs_to :wiki
 
-  validates_presence_of :title, :redirects_to
+  validates_presence_of :wiki_id, :title, :redirects_to
   validates_length_of :title, :redirects_to, :maximum => 255
   attr_protected :id
+
+  before_save :set_redirects_to_wiki_id
+
+  def target_page
+    wiki = Wiki.find_by_id(redirects_to_wiki_id)
+    if wiki
+      wiki.find_page(redirects_to, :with_redirect => false)
+    end
+  end
+
+  private
+
+  def set_redirects_to_wiki_id
+    self.redirects_to_wiki_id ||= wiki_id
+  end
 end
