@@ -26,8 +26,13 @@ Rails.application.configure do
   # Disable request forgery protection in test environment.
   config.action_controller.allow_forgery_protection = false
 
+  # Disable sessions verifications in test environment.
+  config.redmine_verify_sessions = false
+
   # Print deprecation notices to stderr and the Rails logger.
   config.active_support.deprecation = [:stderr, :log]
 
-  config.secret_token = 'a secret token for running the tests'
+  config.secret_key_base = 'a secret token for running the tests'
+
+  config.active_support.test_order = :random
 end
