@@ -5,8 +5,7 @@ module ActiveRecord
     include Redmine::I18n
     # Translate attribute names for validation errors display
     def self.human_attribute_name(attr, *args)
-      attr = attr.to_s.sub(/_id$/, '')
-
+      attr = attr.to_s.sub(/_id$/, '').sub(/^.+\./, '')
       l("field_#{name.underscore.gsub('/', '_')}_#{attr}", :default => ["field_#{attr}".to_sym, attr])
     end
   end
@@ -44,7 +43,7 @@ module ActionView
   class Resolver
     def find_all(name, prefix=nil, partial=false, details={}, key=nil, locals=[])
       cached(key, [name, prefix, partial], details, locals) do
-        if details[:formats] & [:xml, :json]
+        if (details[:formats] & [:xml, :json]).any?
           details = details.dup
           details[:formats] = details[:formats].dup + [:api]
         end
@@ -147,6 +146,16 @@ module ActionMailer
   end
 end
 
+# #deliver is deprecated in Rails 4.2
+# Prevents massive deprecation warnings
+module ActionMailer
+  class MessageDelivery < Delegator
+    def deliver
+      deliver_now
+    end
+  end
+end
+
 module ActionController
   module MimeResponds
     class Collector
@@ -164,45 +173,8 @@ module ActionController
     # TODO: remove it in a later version
     def self.session=(*args)
       $stderr.puts "Please remove config/initializers/session_store.rb and run `rake generate_secret_token`.\n" +
-        "Setting the session secret with ActionController.session= is no longer supported in Rails 3."
+        "Setting the session secret with ActionController.session= is no longer supported."
       exit 1
     end
   end
 end
-
-if Rails::VERSION::MAJOR < 4 && RUBY_VERSION >= "2.1"
-  module ActiveSupport
-    class HashWithIndifferentAccess
-      def select(*args, &block)
-        dup.tap { |hash| hash.select!(*args, &block) }
-      end
-
-      def reject(*args, &block)
-        dup.tap { |hash| hash.reject!(*args, &block) }
-      end
-    end
-
-    class OrderedHash
-      def select(*args, &block)
-        dup.tap { |hash| hash.select!(*args, &block) }
-      end
-
-      def reject(*args, &block)
-        dup.tap { |hash| hash.reject!(*args, &block) }
-      end
-    end
-  end
-end
-
-module CollectiveIdea
-  module Acts
-    module NestedSet
-      module Model
-        def leaf_with_new_record?
-          new_record? || leaf_without_new_record?
-        end
-        alias_method_chain :leaf?, :new_record
-      end
-    end
-  end
-end
