@@ -1,5 +1,5 @@
 # Redmine - project management software
-# Copyright (C) 2006-2014  Jean-Philippe Lang
+# Copyright (C) 2006-2016  Jean-Philippe Lang
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -61,22 +61,30 @@ Rails.application.routes.draw do
   get 'projects/:id/issues/report', :to => 'reports#issue_report', :as => 'project_issues_report'
   get 'projects/:id/issues/report/:detail', :to => 'reports#issue_report_details', :as => 'project_issues_report_details'
 
+  get   '/issues/imports/new', :to => 'imports#new', :as => 'new_issues_import'
+  post  '/imports', :to => 'imports#create', :as => 'imports'
+  get   '/imports/:id', :to => 'imports#show', :as => 'import'
+  match '/imports/:id/settings', :to => 'imports#settings', :via => [:get, :post], :as => 'import_settings'
+  match '/imports/:id/mapping', :to => 'imports#mapping', :via => [:get, :post], :as => 'import_mapping'
+  match '/imports/:id/run', :to => 'imports#run', :via => [:get, :post], :as => 'import_run'
+
   match 'my/account', :controller => 'my', :action => 'account', :via => [:get, :post]
   match 'my/account/destroy', :controller => 'my', :action => 'destroy', :via => [:get, :post]
   match 'my/page', :controller => 'my', :action => 'page', :via => :get
   match 'my', :controller => 'my', :action => 'index', :via => :get # Redirects to my/page
-  match 'my/reset_rss_key', :controller => 'my', :action => 'reset_rss_key', :via => :post
-  match 'my/reset_api_key', :controller => 'my', :action => 'reset_api_key', :via => :post
+  get 'my/api_key', :to => 'my#show_api_key', :as => 'my_api_key'
+  post 'my/api_key', :to => 'my#reset_api_key'
+  post 'my/rss_key', :to => 'my#reset_rss_key', :as => 'my_rss_key'
   match 'my/password', :controller => 'my', :action => 'password', :via => [:get, :post]
   match 'my/page_layout', :controller => 'my', :action => 'page_layout', :via => :get
   match 'my/add_block', :controller => 'my', :action => 'add_block', :via => :post
   match 'my/remove_block', :controller => 'my', :action => 'remove_block', :via => :post
   match 'my/order_blocks', :controller => 'my', :action => 'order_blocks', :via => :post
 
-  resources :users
-  match 'users/:id/memberships/:membership_id', :to => 'users#edit_membership', :via => [:put, :patch], :as => 'user_membership'
-  match 'users/:id/memberships/:membership_id', :to => 'users#destroy_membership', :via => :delete
-  match 'users/:id/memberships', :to => 'users#edit_membership', :via => :post, :as => 'user_memberships'
+  resources :users do
+    resources :memberships, :controller => 'principal_memberships'
+    resources :email_addresses, :only => [:index, :create, :update, :destroy]
+  end
 
   post 'watchers/watch', :to => 'watchers#watch', :as => 'watch'
   delete 'watchers/watch', :to => 'watchers#unwatch'
@@ -111,15 +119,9 @@ Rails.application.routes.draw do
     resource :enumerations, :controller => 'project_enumerations', :only => [:update, :destroy]
 
     get 'issues/:copy_from/copy', :to => 'issues#new', :as => 'copy_issue'
-    resources :issues, :only => [:index, :new, :create] do
-      resources :time_entries, :controller => 'timelog' do
-        collection do
-          get 'report'
-        end
-      end
-    end
-    # issue form update
-    match 'issues/update_form', :controller => 'issues', :action => 'update_form', :via => [:put, :patch, :post], :as => 'issue_form'
+    resources :issues, :only => [:index, :new, :create]
+    # Used when updating the form of a new issue
+    post 'issues/new', :to => 'issues#new'
 
     resources :files, :only => [:index, :new, :create]
 
@@ -133,7 +135,7 @@ Rails.application.routes.draw do
     get 'versions', :to => 'versions#index'
 
     resources :news, :except => [:show, :edit, :update, :destroy]
-    resources :time_entries, :controller => 'timelog' do
+    resources :time_entries, :controller => 'timelog', :except => [:show, :edit, :update, :destroy] do
       get 'report', :on => :collection
     end
     resources :queries, :only => [:new, :create]
@@ -174,11 +176,15 @@ Rails.application.routes.draw do
   end
 
   resources :issues do
+    member do
+      # Used when updating the form of an existing issue
+      patch 'edit', :to => 'issues#edit'
+    end
     collection do
       match 'bulk_edit', :via => [:get, :post]
       post 'bulk_update'
     end
-    resources :time_entries, :controller => 'timelog' do
+    resources :time_entries, :controller => 'timelog', :except => [:show, :edit, :update, :destroy] do
       collection do
         get 'report'
       end
@@ -187,6 +193,8 @@ Rails.application.routes.draw do
       resources :relations, :controller => 'issue_relations', :only => [:index, :show, :create, :destroy]
     end
   end
+  # Used when updating the form of a new issue outside a project
+  post '/issues/new', :to => 'issues#new'
   match '/issues', :controller => 'issues', :action => 'destroy', :via => :delete
 
   resources :queries, :except => [:show]
@@ -215,24 +223,26 @@ Rails.application.routes.draw do
   match '/time_entries/:id', :to => 'timelog#destroy', :via => :delete, :id => /\d+/
   # TODO: delete /time_entries for bulk deletion
   match '/time_entries/destroy', :to => 'timelog#destroy', :via => :delete
+  # Used to update the new time entry form
+  post '/time_entries/new', :to => 'timelog#new'
 
-  get 'projects/:id/activity', :to => 'activities#index'
-  get 'projects/:id/activity.:format', :to => 'activities#index'
+  get 'projects/:id/activity', :to => 'activities#index', :as => :project_activity
   get 'activity', :to => 'activities#index'
 
   # repositories routes
   get 'projects/:id/repository/:repository_id/statistics', :to => 'repositories#stats'
   get 'projects/:id/repository/:repository_id/graph', :to => 'repositories#graph'
 
-  get 'projects/:id/repository/:repository_id/changes(/*path(.:ext))',
-      :to => 'repositories#changes'
+  get 'projects/:id/repository/:repository_id/changes(/*path)',
+      :to => 'repositories#changes',
+      :format => false
 
   get 'projects/:id/repository/:repository_id/revisions/:rev', :to => 'repositories#revision'
   get 'projects/:id/repository/:repository_id/revision', :to => 'repositories#revision'
   post   'projects/:id/repository/:repository_id/revisions/:rev/issues', :to => 'repositories#add_related_issue'
   delete 'projects/:id/repository/:repository_id/revisions/:rev/issues/:issue_id', :to => 'repositories#remove_related_issue'
   get 'projects/:id/repository/:repository_id/revisions', :to => 'repositories#revisions'
-  get 'projects/:id/repository/:repository_id/revisions/:rev/:action(/*path(.:ext))',
+  get 'projects/:id/repository/:repository_id/revisions/:rev/:action(/*path)',
       :controller => 'repositories',
       :format => false,
       :constraints => {
@@ -243,27 +253,30 @@ Rails.application.routes.draw do
   get 'projects/:id/repository/statistics', :to => 'repositories#stats'
   get 'projects/:id/repository/graph', :to => 'repositories#graph'
 
-  get 'projects/:id/repository/changes(/*path(.:ext))',
-      :to => 'repositories#changes'
+  get 'projects/:id/repository/changes(/*path)',
+      :to => 'repositories#changes',
+      :format => false
 
   get 'projects/:id/repository/revisions', :to => 'repositories#revisions'
   get 'projects/:id/repository/revisions/:rev', :to => 'repositories#revision'
   get 'projects/:id/repository/revision', :to => 'repositories#revision'
   post   'projects/:id/repository/revisions/:rev/issues', :to => 'repositories#add_related_issue'
   delete 'projects/:id/repository/revisions/:rev/issues/:issue_id', :to => 'repositories#remove_related_issue'
-  get 'projects/:id/repository/revisions/:rev/:action(/*path(.:ext))',
+  get 'projects/:id/repository/revisions/:rev/:action(/*path)',
       :controller => 'repositories',
       :format => false,
       :constraints => {
             :action => /(browse|show|entry|raw|annotate|diff)/,
             :rev    => /[a-z0-9\.\-_]+/
           }
-  get 'projects/:id/repository/:repository_id/:action(/*path(.:ext))',
+  get 'projects/:id/repository/:repository_id/:action(/*path)',
       :controller => 'repositories',
-      :action => /(browse|show|entry|raw|changes|annotate|diff)/
-  get 'projects/:id/repository/:action(/*path(.:ext))',
+      :action => /(browse|show|entry|raw|changes|annotate|diff)/,
+      :format => false
+  get 'projects/:id/repository/:action(/*path)',
       :controller => 'repositories',
-      :action => /(browse|show|entry|raw|changes|annotate|diff)/
+      :action => /(browse|show|entry|raw|changes|annotate|diff)/,
+      :format => false
 
   get 'projects/:id/repository/:repository_id', :to => 'repositories#show', :path => nil
   get 'projects/:id/repository', :to => 'repositories#show', :path => nil
@@ -274,17 +287,19 @@ Rails.application.routes.draw do
   get 'attachments/download/:id', :to => 'attachments#download', :id => /\d+/
   get 'attachments/thumbnail/:id(/:size)', :to => 'attachments#thumbnail', :id => /\d+/, :size => /\d+/, :as => 'thumbnail'
   resources :attachments, :only => [:show, :destroy]
+  get 'attachments/:object_type/:object_id/edit', :to => 'attachments#edit', :as => :object_attachments_edit
+  patch 'attachments/:object_type/:object_id', :to => 'attachments#update', :as => :object_attachments
 
   resources :groups do
+    resources :memberships, :controller => 'principal_memberships'
     member do
       get 'autocomplete_for_user'
     end
   end
 
-  match 'groups/:id/users', :controller => 'groups', :action => 'add_users', :id => /\d+/, :via => :post, :as => 'group_users'
-  match 'groups/:id/users/:user_id', :controller => 'groups', :action => 'remove_user', :id => /\d+/, :via => :delete, :as => 'group_user'
-  match 'groups/destroy_membership/:id', :controller => 'groups', :action => 'destroy_membership', :id => /\d+/, :via => :post
-  match 'groups/edit_membership/:id', :controller => 'groups', :action => 'edit_membership', :id => /\d+/, :via => :post
+  get 'groups/:id/users/new', :to => 'groups#new_users', :id => /\d+/, :as => 'new_group_users'
+  post 'groups/:id/users', :to => 'groups#add_users', :id => /\d+/, :as => 'group_users'
+  delete 'groups/:id/users/:user_id', :to => 'groups#remove_user', :id => /\d+/, :as => 'group_user'
 
   resources :trackers, :except => :show do
     collection do
@@ -296,7 +311,10 @@ Rails.application.routes.draw do
       post 'update_issue_done_ratio'
     end
   end
-  resources :custom_fields, :except => :show
+  resources :custom_fields, :except => :show do
+    resources :enumerations, :controller => 'custom_field_enumerations', :except => [:show, :new, :edit]
+    put 'enumerations', :to => 'custom_field_enumerations#update_each'
+  end
   resources :roles do
     collection do
       match 'permissions', :via => [:get, :post]
@@ -308,14 +326,16 @@ Rails.application.routes.draw do
   get 'projects/:id/search', :controller => 'search', :action => 'index'
   get 'search', :controller => 'search', :action => 'index'
 
-  match 'mail_handler', :controller => 'mail_handler', :action => 'index', :via => :post
 
-  match 'admin', :controller => 'admin', :action => 'index', :via => :get
-  match 'admin/projects', :controller => 'admin', :action => 'projects', :via => :get
-  match 'admin/plugins', :controller => 'admin', :action => 'plugins', :via => :get
-  match 'admin/info', :controller => 'admin', :action => 'info', :via => :get
-  match 'admin/test_email', :controller => 'admin', :action => 'test_email', :via => :get
-  match 'admin/default_configuration', :controller => 'admin', :action => 'default_configuration', :via => :post
+  get  'mail_handler', :to => 'mail_handler#new'
+  post 'mail_handler', :to => 'mail_handler#index'
+
+  get 'admin', :to => 'admin#index'
+  get 'admin/projects', :to => 'admin#projects'
+  get 'admin/plugins', :to => 'admin#plugins'
+  get 'admin/info', :to => 'admin#info'
+  post 'admin/test_email', :to => 'admin#test_email', :as => 'test_email'
+  post 'admin/default_configuration', :to => 'admin#default_configuration'
 
   resources :auth_sources do
     member do
@@ -336,7 +356,7 @@ Rails.application.routes.draw do
 
   match 'sys/projects', :to => 'sys#projects', :via => :get
   match 'sys/projects/:id/repository', :to => 'sys#create_project_repository', :via => :post
-  match 'sys/fetch_changesets', :to => 'sys#fetch_changesets', :via => :get
+  match 'sys/fetch_changesets', :to => 'sys#fetch_changesets', :via => [:get, :post]
 
   match 'uploads', :to => 'attachments#upload', :via => :post
 
