@@ -7,6 +7,6 @@ class AddRepositoriesChangesPermission < ActiveRecord::Migration
   end
 
   def self.down
-    Permission.find_by_controller_and_action('repositories', 'changes').destroy
+    Permission.where(:controller => "repositories", :action => "changes").each {|p| p.destroy}
   end
 end
