@@ -4,7 +4,7 @@ class AddCustomFieldsPosition < ActiveRecord::Migration
     CustomField.all.group_by(&:type).each  do |t, fields|
       fields.each_with_index do |field, i|
         # do not call model callbacks
-        CustomField.update_all "position = #{i+1}", {:id => field.id}
+        CustomField.where({:id => field.id}).update_all("position = #{i+1}")
       end
     end
   end
