@@ -1,5 +1,5 @@
 # Redmine - project management software
-# Copyright (C) 2006-2014  Jean-Philippe Lang
+# Copyright (C) 2006-2016  Jean-Philippe Lang
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -21,7 +21,9 @@ module Redmine
                   :preview => 'r',
                   :quick_search => 'f',
                   :search => '4',
-                  :new_issue => '7'
+                  :new_issue => '7',
+                  :previous => 'p',
+                  :next => 'n'
                  }.freeze unless const_defined?(:ACCESSKEYS)
 
     def self.key_for(action)
