@@ -1,5 +1,5 @@
 # Redmine - project management software
-# Copyright (C) 2006-2014  Jean-Philippe Lang
+# Copyright (C) 2006-2016  Jean-Philippe Lang
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -21,9 +21,6 @@ module Redmine
     class Fetcher
       attr_reader :user, :project, :scope
 
-      # Needs to be unloaded in development mode
-      @@constantized_providers = Hash.new {|h,k| h[k] = Redmine::Activity.providers[k].collect {|t| t.constantize } }
-
       def initialize(user, options={})
         options.assert_valid_keys(:project, :with_subprojects, :author)
         @user = user
@@ -38,7 +35,25 @@ module Redmine
         return @event_types unless @event_types.nil?
 
         @event_types = Redmine::Activity.available_event_types
-        @event_types = @event_types.select {|o| @project.self_and_descendants.detect {|p| @user.allowed_to?("view_#{o}".to_sym, p)}} if @project
+        if @project
+          projects = @project.self_and_descendants
+          @event_types = @event_types.select do |event_type|
+            keep = false
+            constantized_providers(event_type).each do |provider|
+              options = provider.activity_provider_options[event_type]
+              permission = options[:permission]
+              unless options.key?(:permission)
+                permission ||= "view_#{event_type}".to_sym
+              end
+              if permission
+                keep |= projects.any? {|p| @user.allowed_to?(permission, p)}
+              else
+                keep = true
+              end
+            end
+            keep
+          end
+        end
         @event_types
       end
 
@@ -88,7 +103,7 @@ module Redmine
       private
 
       def constantized_providers(event_type)
-        @@constantized_providers[event_type]
+        Redmine::Activity.providers[event_type].map(&:constantize)
       end
     end
   end
