@@ -1,5 +1,5 @@
 # Redmine - project management software
-# Copyright (C) 2006-2014  Jean-Philippe Lang
+# Copyright (C) 2006-2016  Jean-Philippe Lang
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -61,6 +61,10 @@ module Redmine
       class_attribute :searchable_supported
       self.searchable_supported = false
 
+      # Set this to true if field values can be summed up
+      class_attribute :totalable_supported
+      self.totalable_supported = false
+
       # Restricts the classes that the custom field can be added to
       # Set to nil for no restrictions
       class_attribute :customized_class_names
@@ -70,6 +74,9 @@ module Redmine
       class_attribute :form_partial
       self.form_partial = nil
 
+      class_attribute :change_as_diff
+      self.change_as_diff = false
+
       def self.add(name)
         self.format_name = name
         Redmine::FieldFormat.add(name, self)
@@ -123,6 +130,22 @@ module Redmine
         []
       end
 
+      def value_from_keyword(custom_field, keyword, object)
+        possible_values_options = possible_values_options(custom_field, object)
+        if possible_values_options.present?
+          keyword = keyword.to_s
+          if v = possible_values_options.detect {|text, id| keyword.casecmp(text)  == 0}
+            if v.is_a?(Array)
+              v.last
+            else
+              v
+            end
+          end
+        else
+          keyword
+        end
+      end
+
       # Returns the validation errors for custom_field
       # Should return an empty array if custom_field is valid
       def validate_custom_field(custom_field)
@@ -293,13 +316,18 @@ module Redmine
       add 'text'
       self.searchable_supported = true
       self.form_partial = 'custom_fields/formats/text'
+      self.change_as_diff = true
 
       def formatted_value(view, custom_field, value, customized=nil, html=false)
         if html
-          if custom_field.text_formatting == 'full'
-            view.textilizable(value, :object => customized)
+          if value.present?
+            if custom_field.text_formatting == 'full'
+              view.textilizable(value, :object => customized)
+            else
+              view.simple_format(html_escape(value))
+            end
           else
-            view.simple_format(html_escape(value))
+            ''
           end
         else
           value.to_s
@@ -337,7 +365,7 @@ module Redmine
               url = "http://" + url
             end
           end
-          view.link_to value.to_s, url
+          view.link_to value.to_s.truncate(40), url
         else
           value.to_s
         end
@@ -346,6 +374,7 @@ module Redmine
 
     class Numeric < Unbounded
       self.form_partial = 'custom_fields/formats/numeric'
+      self.totalable_supported = true
 
       def order_statement(custom_field)
         # Make the database cast values into numeric
@@ -353,6 +382,18 @@ module Redmine
         # CustomValue validations should ensure that it doesn't occur
         "CAST(CASE #{join_alias custom_field}.value WHEN '' THEN '0' ELSE #{join_alias custom_field}.value END AS decimal(30,3))"
       end
+
+      # Returns totals for the given scope
+      def total_for_scope(custom_field, scope)
+        scope.joins(:custom_values).
+          where(:custom_values => {:custom_field_id => custom_field.id}).
+          where.not(:custom_values => {:value => ''}).
+          sum("CAST(#{CustomValue.table_name}.value AS decimal(30,3))")
+      end
+
+      def cast_total_value(custom_field, value)
+        cast_single_value(custom_field, value)
+      end
     end
 
     class IntFormat < Numeric
@@ -368,7 +409,7 @@ module Redmine
 
       def validate_single_value(custom_field, value, customized=nil)
         errs = super
-        errs << ::I18n.t('activerecord.errors.messages.not_a_number') unless value =~ /^[+-]?\d+$/
+        errs << ::I18n.t('activerecord.errors.messages.not_a_number') unless value.to_s =~ /^[+-]?\d+$/
         errs
       end
 
@@ -388,6 +429,10 @@ module Redmine
         value.to_f
       end
 
+      def cast_total_value(custom_field, value)
+        value.to_f.round(2)
+      end
+
       def validate_single_value(custom_field, value, customized=nil)
         errs = super
         errs << ::I18n.t('activerecord.errors.messages.invalid') unless (Kernel.Float(value) rescue nil)
@@ -456,11 +501,16 @@ module Redmine
       end
 
       def query_filter_options(custom_field, query)
-        {:type => :list_optional, :values => possible_values_options(custom_field, query.project)}
+        {:type => :list_optional, :values => query_filter_values(custom_field, query)}
       end
 
       protected
 
+      # Returns the values that are available in the field filter
+      def query_filter_values(custom_field, query)
+        possible_values_options(custom_field, query.project)
+      end
+
       # Renders the edit tag as a select tag
       def select_edit_tag(view, tag_id, tag_name, custom_value, options={})
         blank_option = ''.html_safe
@@ -510,7 +560,7 @@ module Redmine
       add 'list'
       self.searchable_supported = true
       self.form_partial = 'custom_fields/formats/list'
- 
+
       def possible_custom_value_options(custom_value)
         options = possible_values_options(custom_value.custom_field)
         missing = [custom_value.value].flatten.reject(&:blank?) - options
@@ -588,7 +638,7 @@ module Redmine
     end
 
     class RecordList < List
-      self.customized_class_names = %w(Issue TimeEntry Version Project)
+      self.customized_class_names = %w(Issue TimeEntry Version Document Project)
 
       def cast_single_value(custom_field, value, customized=nil)
         target_class.find_by_id(value.to_i) if value.present?
@@ -597,13 +647,16 @@ module Redmine
       def target_class
         @target_class ||= self.class.name[/^(.*::)?(.+)Format$/, 2].constantize rescue nil
       end
+
+      def reset_target_class
+        @target_class = nil
+      end
  
       def possible_custom_value_options(custom_value)
         options = possible_values_options(custom_value.custom_field, custom_value.customized)
         missing = [custom_value.value_was].flatten.reject(&:blank?) - options.map(&:last)
         if missing.any?
           options += target_class.where(:id => missing.map(&:to_i)).map {|o| [o.to_s, o.id.to_s]}
-          options.sort_by!(&:first)
         end
         options
       end
@@ -641,15 +694,45 @@ module Redmine
       protected :value_join_alias
     end
 
+    class EnumerationFormat < RecordList
+      add 'enumeration'
+      self.form_partial = 'custom_fields/formats/enumeration'
+ 
+      def label
+        "label_field_format_enumeration"
+      end
+
+      def target_class
+        @target_class ||= CustomFieldEnumeration
+      end
+
+      def possible_values_options(custom_field, object=nil)
+        possible_values_records(custom_field, object).map {|u| [u.name, u.id.to_s]}
+      end
+
+      def possible_values_records(custom_field, object=nil)
+        custom_field.enumerations.active
+      end
+
+      def value_from_keyword(custom_field, keyword, object)
+        value = custom_field.enumerations.where("LOWER(name) LIKE LOWER(?)", keyword)
+        value ? value.id : nil
+      end
+    end
+
     class UserFormat < RecordList
       add 'user'
       self.form_partial = 'custom_fields/formats/user'
       field_attributes :user_role
 
       def possible_values_options(custom_field, object=nil)
+        possible_values_records(custom_field, object).map {|u| [u.name, u.id.to_s]}
+      end
+
+      def possible_values_records(custom_field, object=nil)
         if object.is_a?(Array)
           projects = object.map {|o| o.respond_to?(:project) ? o.project : nil}.compact.uniq
-          projects.map {|project| possible_values_options(custom_field, project)}.reduce(:&) || []
+          projects.map {|project| possible_values_records(custom_field, project)}.reduce(:&) || []
         elsif object.respond_to?(:project) && object.project
           scope = object.project.users
           if custom_field.user_role.is_a?(Array)
@@ -658,12 +741,18 @@ module Redmine
               scope = scope.where("#{Member.table_name}.id IN (SELECT DISTINCT member_id FROM #{MemberRole.table_name} WHERE role_id IN (?))", role_ids)
             end
           end
-          scope.sorted.collect {|u| [u.to_s, u.id.to_s]}
+          scope.sorted
         else
           []
         end
       end
 
+      def value_from_keyword(custom_field, keyword, object)
+        users = possible_values_records(custom_field, object).to_a
+        user = Principal.detect_by_keyword(users, keyword)
+        user ? user.id : nil
+      end
+
       def before_custom_field_save(custom_field)
         super
         if custom_field.user_role.is_a?(Array)
@@ -678,12 +767,29 @@ module Redmine
       field_attributes :version_status
 
       def possible_values_options(custom_field, object=nil)
+        versions_options(custom_field, object)
+      end
+
+      def before_custom_field_save(custom_field)
+        super
+        if custom_field.version_status.is_a?(Array)
+          custom_field.version_status.map!(&:to_s).reject!(&:blank?)
+        end
+      end
+
+      protected
+
+      def query_filter_values(custom_field, query)
+        versions_options(custom_field, query.project, true)
+      end
+
+      def versions_options(custom_field, object, all_statuses=false)
         if object.is_a?(Array)
           projects = object.map {|o| o.respond_to?(:project) ? o.project : nil}.compact.uniq
           projects.map {|project| possible_values_options(custom_field, project)}.reduce(:&) || []
         elsif object.respond_to?(:project) && object.project
           scope = object.project.shared_versions
-          if custom_field.version_status.is_a?(Array)
+          if !all_statuses && custom_field.version_status.is_a?(Array)
             statuses = custom_field.version_status.map(&:to_s).reject(&:blank?)
             if statuses.any?
               scope = scope.where(:status => statuses.map(&:to_s))
@@ -694,13 +800,6 @@ module Redmine
           []
         end
       end
-
-      def before_custom_field_save(custom_field)
-        super
-        if custom_field.version_status.is_a?(Array)
-          custom_field.version_status.map!(&:to_s).reject!(&:blank?)
-        end
-      end
     end
   end
 end
