@@ -1,5 +1,5 @@
 # Redmine - project management software
-# Copyright (C) 2006-2014  Jean-Philippe Lang
+# Copyright (C) 2006-2016  Jean-Philippe Lang
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -283,7 +283,7 @@ module Redmine
 
         def scm_iconv(to, from, str)
           return nil if str.nil?
-          return str if to == from
+          return str if to == from && str.encoding.to_s == from
           str.force_encoding(from)
           begin
             str.encode(to)
