@@ -1,5 +1,5 @@
 # Redmine - project management software
-# Copyright (C) 2006-2014  Jean-Philippe Lang
+# Copyright (C) 2006-2016  Jean-Philippe Lang
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -51,7 +51,7 @@ class ActivitiesControllerTest < ActionController::TestCase
     assert_not_nil assigns(:events_by_day)
 
     assert_select 'h3', :text => /#{3.days.ago.to_date.day}/
-    assert_select 'dl dt.issue a', :text => /#{ESCAPED_UCANT} print recipes/
+    assert_select 'dl dt.issue a', :text => /Cannot print recipes/
   end
 
   def test_global_index
@@ -75,13 +75,13 @@ class ActivitiesControllerTest < ActionController::TestCase
     assert_template 'index'
     assert_not_nil assigns(:events_by_day)
 
-    assert_select 'h2 a[href=/users/2]', :text => 'John Smith'
+    assert_select 'h2 a[href="/users/2"]', :text => 'John Smith'
 
     i1 = Issue.find(1)
     d1 = User.find(1).time_to_date(i1.created_on)
 
     assert_select 'h3', :text => /#{d1.day}/
-    assert_select 'dl dt.issue a', :text => /#{ESCAPED_UCANT} print recipes/
+    assert_select 'dl dt.issue a', :text => /Cannot print recipes/
   end
 
   def test_user_index_with_invalid_user_id_should_respond_404
@@ -118,8 +118,8 @@ class ActivitiesControllerTest < ActionController::TestCase
     assert_template 'common/feed'
 
     assert_select 'feed' do
-      assert_select 'link[rel=self][href=?]', 'http://test.host/activity.atom?show_changesets=1&amp;show_documents=1&amp;show_files=1&amp;show_issues=1&amp;show_messages=1&amp;show_news=1&amp;show_time_entries=1&amp;show_wiki_edits=1&amp;with_subprojects=0'
-      assert_select 'link[rel=alternate][href=?]', 'http://test.host/activity?show_changesets=1&amp;show_documents=1&amp;show_files=1&amp;show_issues=1&amp;show_messages=1&amp;show_news=1&amp;show_time_entries=1&amp;show_wiki_edits=1&amp;with_subprojects=0'
+      assert_select 'link[rel=self][href=?]', 'http://test.host/activity.atom?show_changesets=1&show_documents=1&show_files=1&show_issues=1&show_messages=1&show_news=1&show_time_entries=1&show_wiki_edits=1&with_subprojects=0'
+      assert_select 'link[rel=alternate][href=?]', 'http://test.host/activity?show_changesets=1&show_documents=1&show_files=1&show_issues=1&show_messages=1&show_news=1&show_time_entries=1&show_wiki_edits=1&with_subprojects=0'
       assert_select 'entry' do
         assert_select 'link[href=?]', 'http://test.host/issues/11'
       end
@@ -136,6 +136,14 @@ class ActivitiesControllerTest < ActionController::TestCase
     end
   end
 
+  def test_index_atom_feed_with_user
+    get :index, :user_id => 2, :format => 'atom'
+
+    assert_response :success
+    assert_template 'common/feed'
+    assert_select 'title', :text => "Redmine: #{User.find(2).name}"
+  end
+
   def test_index_should_show_private_notes_with_permission_only
     journal = Journal.create!(:journalized => Issue.find(2), :notes => 'Private notes with searchkeyword', :private_notes => true)
     @request.session[:user_id] = 2
@@ -149,4 +157,23 @@ class ActivitiesControllerTest < ActionController::TestCase
     assert_response :success
     assert_not_include journal, assigns(:events_by_day).values.flatten
   end
+
+  def test_index_with_submitted_scope_should_save_as_preference
+    @request.session[:user_id] = 2
+
+    get :index, :show_issues => '1', :show_messages => '1', :submit => 'Apply'
+    assert_response :success
+    assert_equal %w(issues messages), User.find(2).pref.activity_scope.sort
+  end
+
+  def test_index_scope_should_default_to_user_preference
+    pref = User.find(2).pref
+    pref.activity_scope = %w(issues news)
+    pref.save!
+    @request.session[:user_id] = 2
+
+    get :index
+    assert_response :success
+    assert_equal %w(issues news), assigns(:activity).scope
+  end
 end
