@@ -1,5 +1,5 @@
 # Redmine - project management software
-# Copyright (C) 2006-2014  Jean-Philippe Lang
+# Copyright (C) 2006-2016  Jean-Philippe Lang
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -18,7 +18,7 @@
 require File.expand_path('../../test_helper', __FILE__)
 
 class AdminControllerTest < ActionController::TestCase
-  fixtures :projects, :users, :roles
+  fixtures :projects, :users, :email_addresses, :roles
 
   def setup
     User.current = nil
@@ -87,7 +87,7 @@ class AdminControllerTest < ActionController::TestCase
     user.pref.save!
     ActionMailer::Base.deliveries.clear
 
-    get :test_email
+    post :test_email
     assert_redirected_to '/settings?tab=notifications'
     mail = ActionMailer::Base.deliveries.last
     assert_not_nil mail
@@ -97,7 +97,7 @@ class AdminControllerTest < ActionController::TestCase
 
   def test_test_email_failure_should_display_the_error
     Mailer.stubs(:test_email).raises(Exception, 'Some error message')
-    get :test_email
+    post :test_email
     assert_redirected_to '/settings?tab=notifications'
     assert_match /Some error message/, flash[:error]
   end
@@ -129,7 +129,7 @@ class AdminControllerTest < ActionController::TestCase
 
     assert_select 'tr#plugin-foo' do
       assert_select 'td span.name', :text => 'Foo plugin'
-      assert_select 'td.configure a[href=/settings/plugin/foo]'
+      assert_select 'td.configure a[href="/settings/plugin/foo"]'
     end
     assert_select 'tr#plugin-bar' do
       assert_select 'td span.name', :text => 'Bar'
@@ -150,7 +150,7 @@ class AdminControllerTest < ActionController::TestCase
 
     get :index
     assert_response :success
-    assert_select 'div#admin-menu a[href=/foo/bar]', :text => 'Test'
+    assert_select 'div#admin-menu a[href="/foo/bar"]', :text => 'Test'
 
     Redmine::MenuManager.map :admin_menu do |menu|
       menu.delete :test_admin_menu_plugin_extension
