@@ -1,5 +1,5 @@
 # Redmine - project management software
-# Copyright (C) 2006-2014  Jean-Philippe Lang
+# Copyright (C) 2006-2016  Jean-Philippe Lang
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -24,7 +24,12 @@ class CalendarsControllerTest < ActionController::TestCase
            :roles,
            :member_roles,
            :members,
-           :enabled_modules
+           :enabled_modules,
+           :issues,
+           :issue_statuses,
+           :issue_relations,
+           :issue_categories,
+           :enumerations
 
   def test_show
     get :show, :project_id => 1
