@@ -1,5 +1,5 @@
 # Redmine - project management software
-# Copyright (C) 2006-2014  Jean-Philippe Lang
+# Copyright (C) 2006-2016  Jean-Philippe Lang
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -47,6 +47,14 @@ class IssueRelationsControllerTest < ActionController::TestCase
     assert_equal 'relates', relation.relation_type
   end
 
+  def test_create_on_invalid_issue
+    assert_no_difference 'IssueRelation.count' do
+      post :create, :issue_id => 999,
+        :relation => {:issue_to_id => '2', :relation_type => 'relates', :delay => ''}
+      assert_response 404
+    end
+  end
+
   def test_create_xhr
     assert_difference 'IssueRelation.count' do
       xhr :post, :create, :issue_id => 3, :relation => {:issue_to_id => '1', :relation_type => 'relates', :delay => ''}
@@ -127,6 +135,13 @@ class IssueRelationsControllerTest < ActionController::TestCase
     end
   end
 
+  def test_destroy_invalid_relation
+    assert_no_difference 'IssueRelation.count' do
+      delete :destroy, :id => '999'
+      assert_response 404
+    end
+  end
+
   def test_destroy_xhr
     IssueRelation.create!(:relation_type => IssueRelation::TYPE_RELATES) do |r|
       r.issue_from_id = 3
