@@ -1,5 +1,5 @@
 # Redmine - project management software
-# Copyright (C) 2006-2014  Jean-Philippe Lang
+# Copyright (C) 2006-2016  Jean-Philippe Lang
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -18,7 +18,7 @@
 require File.expand_path('../../test_helper', __FILE__)
 
 class IssueStatusesControllerTest < ActionController::TestCase
-  fixtures :issue_statuses, :issues, :users
+  fixtures :issue_statuses, :issues, :users, :trackers
 
   def setup
     User.current = nil
@@ -62,7 +62,7 @@ class IssueStatusesControllerTest < ActionController::TestCase
     post :create, :issue_status => {:name => ''}
     assert_response :success
     assert_template 'new'
-    assert_error_tag :content => /name #{ESCAPED_CANT} be blank/i
+    assert_select_error /name cannot be blank/i
   end
 
   def test_edit
@@ -82,11 +82,12 @@ class IssueStatusesControllerTest < ActionController::TestCase
     put :update, :id => '3', :issue_status => {:name => ''}
     assert_response :success
     assert_template 'edit'
-    assert_error_tag :content => /name #{ESCAPED_CANT} be blank/i
+    assert_select_error /name cannot be blank/i
   end
 
   def test_destroy
-    Issue.delete_all("status_id = 1")
+    Issue.where(:status_id => 1).delete_all
+    Tracker.where(:default_status_id => 1).delete_all
 
     assert_difference 'IssueStatus.count', -1 do
       delete :destroy, :id => '1'
@@ -95,8 +96,20 @@ class IssueStatusesControllerTest < ActionController::TestCase
     assert_nil IssueStatus.find_by_id(1)
   end
 
-  def test_destroy_should_block_if_status_in_use
-    assert_not_nil Issue.find_by_status_id(1)
+  def test_destroy_should_block_if_status_is_used_by_issues
+    assert Issue.where(:status_id => 1).any?
+    Tracker.where(:default_status_id => 1).delete_all
+
+    assert_no_difference 'IssueStatus.count' do
+      delete :destroy, :id => '1'
+    end
+    assert_redirected_to :action => 'index'
+    assert_not_nil IssueStatus.find_by_id(1)
+  end
+
+  def test_destroy_should_block_if_status_is_used_as_tracker_default_status
+    Issue.where(:status_id => 1).delete_all
+    assert Tracker.where(:default_status_id => 1).any?
 
     assert_no_difference 'IssueStatus.count' do
       delete :destroy, :id => '1'
