@@ -1,5 +1,5 @@
 # Redmine - project management software
-# Copyright (C) 2006-2014  Jean-Philippe Lang
+# Copyright (C) 2006-2016  Jean-Philippe Lang
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -75,7 +75,7 @@ class IssuesControllerTransactionTest < ActionController::TestCase
     assert_select 'input[name=?][value=?]', 'conflict_resolution', 'add_notes'
     assert_select 'label' do
       assert_select 'input[name=?][value=?]', 'conflict_resolution', 'cancel'
-      assert_select 'a[href=/issues/2]'
+      assert_select 'a[href="/issues/2"]'
     end
   end
 
@@ -103,8 +103,8 @@ class IssuesControllerTransactionTest < ActionController::TestCase
     assert_response :success
     assert_template 'edit'
     attachment = Attachment.order('id DESC').first
-    assert_tag 'input', :attributes => {:name => 'attachments[p0][token]', :value => attachment.token}
-    assert_tag 'input', :attributes => {:name => 'attachments[p0][filename]', :value => 'testfile.txt'}
+    assert_select 'input[name=?][value=?]', 'attachments[p0][token]', attachment.token
+    assert_select 'input[name=?][value=?]', 'attachments[p0][filename]', 'testfile.txt'
   end
 
   def test_update_stale_issue_without_notes_should_not_show_add_notes_option
@@ -118,10 +118,10 @@ class IssuesControllerTransactionTest < ActionController::TestCase
             :lock_version => (issue.lock_version - 1)
           }
 
-    assert_tag 'div', :attributes => {:class => 'conflict'}
-    assert_tag 'input', :attributes => {:name => 'conflict_resolution', :value => 'overwrite'}
-    assert_no_tag 'input', :attributes => {:name => 'conflict_resolution', :value => 'add_notes'}
-    assert_tag 'input', :attributes => {:name => 'conflict_resolution', :value => 'cancel'}
+    assert_select 'div.conflict'
+    assert_select 'input[name=conflict_resolution][value=overwrite]'
+    assert_select 'input[name=conflict_resolution][value=add_notes]', 0
+    assert_select 'input[name=conflict_resolution][value=cancel]'
   end
 
   def test_update_stale_issue_should_show_conflicting_journals
@@ -138,8 +138,8 @@ class IssuesControllerTransactionTest < ActionController::TestCase
     assert_not_nil assigns(:conflict_journals)
     assert_equal 1, assigns(:conflict_journals).size
     assert_equal 2, assigns(:conflict_journals).first.id
-    assert_tag 'div', :attributes => {:class => 'conflict'},
-      :descendant => {:content => /Some notes with Redmine links/}
+
+    assert_select 'div.conflict', :text => /Some notes with Redmine links/
   end
 
   def test_update_stale_issue_without_previous_journal_should_show_all_journals
@@ -155,10 +155,8 @@ class IssuesControllerTransactionTest < ActionController::TestCase
 
     assert_not_nil assigns(:conflict_journals)
     assert_equal 2, assigns(:conflict_journals).size
-    assert_tag 'div', :attributes => {:class => 'conflict'},
-      :descendant => {:content => /Some notes with Redmine links/}
-    assert_tag 'div', :attributes => {:class => 'conflict'},
-      :descendant => {:content => /Journal notes/}
+    assert_select 'div.conflict', :text => /Some notes with Redmine links/
+    assert_select 'div.conflict', :text => /Journal notes/
   end
 
   def test_update_stale_issue_should_show_private_journals_with_permission_only
@@ -212,6 +210,27 @@ class IssuesControllerTransactionTest < ActionController::TestCase
     assert_nil issue.fixed_version_id
     journal = Journal.order('id DESC').first
     assert_equal 'add_notes_conflict_resolution', journal.notes
+    assert_equal false, journal.private_notes
+    assert journal.details.empty?
+  end
+
+  def test_update_stale_issue_with_add_notes_conflict_resolution_should_preserve_private_notes
+    @request.session[:user_id] = 2
+
+    journal = new_record(Journal) do
+      put :update, :id => 1,
+            :issue => {
+              :fixed_version_id => 4,
+              :notes => 'add_privates_notes_conflict_resolution',
+              :private_notes => '1',
+              :lock_version => 2
+            },
+            :conflict_resolution => 'add_notes'
+    end
+
+    assert_response 302
+    assert_equal 'add_privates_notes_conflict_resolution', journal.notes
+    assert_equal true, journal.private_notes
     assert journal.details.empty?
   end
 
@@ -247,7 +266,7 @@ class IssuesControllerTransactionTest < ActionController::TestCase
     assert_select 'input[name=?][value=?]', 'time_entry[hours]', '2.5'
     assert_select 'input[name=?][value=?]', 'time_entry[comments]', 'should not be added'
     assert_select 'select[name=?]', 'time_entry[activity_id]' do
-      assert_select 'option[value=?][selected=selected]', TimeEntryActivity.first.id
+      assert_select 'option[value=?][selected=selected]', TimeEntryActivity.first.id.to_s
     end
   end
 
@@ -256,7 +275,7 @@ class IssuesControllerTransactionTest < ActionController::TestCase
 
     get :index
     assert_response 500
-    assert_tag 'p', :content => /An error occurred/
+    assert_select 'p', :text => /An error occurred/
     assert_nil session[:query]
     assert_nil session[:issues_index_sort]
   end
