@@ -1,5 +1,5 @@
 # Redmine - project management software
-# Copyright (C) 2006-2014  Jean-Philippe Lang
+# Copyright (C) 2006-2016  Jean-Philippe Lang
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -19,7 +19,7 @@ require File.expand_path('../../test_helper', __FILE__)
 
 class JournalsControllerTest < ActionController::TestCase
   fixtures :projects, :users, :members, :member_roles, :roles, :issues, :journals, :journal_details, :enabled_modules,
-    :trackers, :issue_statuses, :enumerations, :custom_fields, :custom_values, :custom_fields_projects
+    :trackers, :issue_statuses, :enumerations, :custom_fields, :custom_values, :custom_fields_projects, :projects_trackers
 
   def setup
     User.current = nil
@@ -32,6 +32,11 @@ class JournalsControllerTest < ActionController::TestCase
     assert_equal 'application/atom+xml', @response.content_type
   end
 
+  def test_index_with_invalid_query_id
+    get :index, :project_id => 1, :query_id => 999
+    assert_response 404
+  end
+
   def test_index_should_return_privates_notes_with_permission_only
     journal = Journal.create!(:journalized => Issue.find(2), :notes => 'Privates notes', :private_notes => true, :user_id => 1)
     @request.session[:user_id] = 2
@@ -46,17 +51,86 @@ class JournalsControllerTest < ActionController::TestCase
     assert_not_include journal, assigns(:journals)
   end
 
-  def test_diff
+  def test_index_should_show_visible_custom_fields_only
+    Issue.destroy_all
+    field_attributes = {:field_format => 'string', :is_for_all => true, :is_filter => true, :trackers => Tracker.all}
+    @fields = []
+    @fields << (@field1 = IssueCustomField.create!(field_attributes.merge(:name => 'Field 1', :visible => true)))
+    @fields << (@field2 = IssueCustomField.create!(field_attributes.merge(:name => 'Field 2', :visible => false, :role_ids => [1, 2])))
+    @fields << (@field3 = IssueCustomField.create!(field_attributes.merge(:name => 'Field 3', :visible => false, :role_ids => [1, 3])))
+    @issue = Issue.generate!(
+      :author_id => 1,
+      :project_id => 1,
+      :tracker_id => 1,
+      :custom_field_values => {@field1.id => 'Value0', @field2.id => 'Value1', @field3.id => 'Value2'}
+    )
+    @issue.init_journal(User.find(1))
+    @issue.update_attribute :custom_field_values, {@field1.id => 'NewValue0', @field2.id => 'NewValue1', @field3.id => 'NewValue2'}
+
+
+    user_with_role_on_other_project = User.generate!
+    User.add_to_project(user_with_role_on_other_project, Project.find(2), Role.find(3))
+    users_to_test = {
+      User.find(1) => [@field1, @field2, @field3],
+      User.find(3) => [@field1, @field2],
+      user_with_role_on_other_project => [@field1], # should see field1 only on Project 1
+      User.generate! => [@field1],
+      User.anonymous => [@field1]
+    }
+
+    users_to_test.each do |user, visible_fields|
+      get :index, :format => 'atom', :key => user.rss_key
+      @fields.each_with_index do |field, i|
+        if visible_fields.include?(field)
+          assert_select "content[type=html]", { :text => /NewValue#{i}/, :count => 1 }, "User #{user.id} was not able to view #{field.name} in API"
+        else
+          assert_select "content[type=html]", { :text => /NewValue#{i}/, :count => 0 }, "User #{user.id} was able to view #{field.name} in API"
+        end
+      end
+    end
+
+  end
+
+  def test_diff_for_description_change
     get :diff, :id => 3, :detail_id => 4
     assert_response :success
     assert_template 'diff'
 
-    assert_tag 'span',
-      :attributes => {:class => 'diff_out'},
-      :content => /removed/
-    assert_tag 'span',
-      :attributes => {:class => 'diff_in'},
-      :content => /added/
+    assert_select 'span.diff_out', :text => /removed/
+    assert_select 'span.diff_in', :text => /added/
+  end
+
+  def test_diff_for_custom_field
+    field = IssueCustomField.create!(:name => "Long field", :field_format => 'text')
+    journal = Journal.create!(:journalized => Issue.find(2), :notes => 'Notes', :user_id => 1)
+    detail = JournalDetail.create!(:journal => journal, :property => 'cf', :prop_key => field.id,
+      :old_value => 'Foo', :value => 'Bar')
+
+    get :diff, :id => journal.id, :detail_id => detail.id
+    assert_response :success
+    assert_template 'diff'
+
+    assert_select 'span.diff_out', :text => /Foo/
+    assert_select 'span.diff_in', :text => /Bar/
+  end
+
+  def test_diff_for_custom_field_should_be_denied_if_custom_field_is_not_visible
+    field = IssueCustomField.create!(:name => "Long field", :field_format => 'text', :visible => false, :role_ids => [1])
+    journal = Journal.create!(:journalized => Issue.find(2), :notes => 'Notes', :user_id => 1)
+    detail = JournalDetail.create!(:journal => journal, :property => 'cf', :prop_key => field.id,
+      :old_value => 'Foo', :value => 'Bar')
+
+    get :diff, :id => journal.id, :detail_id => detail.id
+    assert_response 302
+  end
+
+  def test_diff_should_default_to_description_diff
+    get :diff, :id => 3
+    assert_response :success
+    assert_template 'diff'
+
+    assert_select 'span.diff_out', :text => /removed/
+    assert_select 'span.diff_in', :text => /added/
   end
 
   def test_reply_to_issue
