@@ -1,5 +1,5 @@
 # Redmine - project management software
-# Copyright (C) 2006-2014  Jean-Philippe Lang
+# Copyright (C) 2006-2016  Jean-Philippe Lang
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -51,7 +51,7 @@ class PreviewsControllerTest < ActionController::TestCase
     assert_response :success
     assert_template 'previews/issue'
     assert_not_nil assigns(:notes)
-    assert_tag :p, :content => 'Foo'
+    assert_select 'p', :text => 'Foo'
   end
 
   def test_preview_issue_notes_should_support_links_to_existing_attachments
@@ -62,6 +62,14 @@ class PreviewsControllerTest < ActionController::TestCase
     assert_select 'a.attachment', :text => 'foo.bar'
   end
 
+  def test_preview_issue_with_project_changed
+    @request.session[:user_id] = 2
+    post :issue, :project_id => '1', :id => 1, :issue => {:notes => 'notes', :project_id => 2}
+    assert_response :success
+    assert_not_nil assigns(:issue)
+    assert_not_nil assigns(:notes)
+  end
+
   def test_preview_new_news
     get :news, :project_id => 1,
                   :news => {:title => '',
@@ -69,8 +77,7 @@ class PreviewsControllerTest < ActionController::TestCase
                             :summary => ''}
     assert_response :success
     assert_template 'common/_preview'
-    assert_tag :tag => 'fieldset', :attributes => { :class => 'preview' },
-                                   :content => /News description/
+    assert_select 'fieldset.preview', :text => /News description/
   end
 
   def test_existing_new_news
@@ -83,7 +90,6 @@ class PreviewsControllerTest < ActionController::TestCase
     assert_equal News.find(2), assigns(:previewed)
     assert_not_nil assigns(:attachments)
 
-    assert_tag :tag => 'fieldset', :attributes => { :class => 'preview' },
-                                   :content => /News description/
+    assert_select 'fieldset.preview', :text => /News description/
   end
 end
