@@ -1,5 +1,5 @@
 # Redmine - project management software
-# Copyright (C) 2006-2014  Jean-Philippe Lang
+# Copyright (C) 2006-2016  Jean-Philippe Lang
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -51,7 +51,7 @@ class WikisControllerTest < ActionController::TestCase
     end
 
     assert_include 'errorExplanation', response.body
-    assert_include "Start page #{ESCAPED_CANT} be blank", response.body
+    assert_include "Start page cannot be blank", response.body
   end
 
   def test_update
