@@ -1,5 +1,5 @@
 # Redmine - project management software
-# Copyright (C) 2006-2014  Jean-Philippe Lang
+# Copyright (C) 2006-2016  Jean-Philippe Lang
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -31,8 +31,7 @@ class WorkflowsControllerTest < ActionController::TestCase
     assert_template 'index'
 
     count = WorkflowTransition.where(:role_id => 1, :tracker_id => 2).count
-    assert_tag :tag => 'a', :content => count.to_s,
-                            :attributes => { :href => '/workflows/edit?role_id=1&amp;tracker_id=2' }
+    assert_select 'a[href=?]', '/workflows/edit?role_id=1&tracker_id=2', :content => count.to_s
   end
 
   def test_get_edit
@@ -55,13 +54,30 @@ class WorkflowsControllerTest < ActionController::TestCase
     assert_equal [2, 3, 5], assigns(:statuses).collect(&:id)
 
     # allowed transitions
-    assert_select 'input[type=checkbox][name=?][value=1][checked=checked]', 'transitions[3][5][always]'
+    assert_select 'input[type=checkbox][name=?][value="1"][checked=checked]', 'transitions[3][5][always]'
     # not allowed
-    assert_select 'input[type=checkbox][name=?][value=1]:not([checked=checked])', 'transitions[3][2][always]'
+    assert_select 'input[type=checkbox][name=?][value="1"]:not([checked=checked])', 'transitions[3][2][always]'
     # unused
     assert_select 'input[type=checkbox][name=?]', 'transitions[1][1][always]', 0
   end
 
+  def test_get_edit_should_include_allowed_statuses_for_new_issues
+    WorkflowTransition.delete_all
+    WorkflowTransition.create!(:role_id => 1, :tracker_id => 1, :old_status_id => 0, :new_status_id => 1)
+
+    get :edit, :role_id => 1, :tracker_id => 1
+    assert_response :success
+    assert_select 'td', 'New issue'
+    assert_select 'input[type=checkbox][name=?][value="1"][checked=checked]', 'transitions[0][1][always]'
+  end
+
+  def test_get_edit_with_all_roles_and_all_trackers
+    get :edit, :role_id => 'all', :tracker_id => 'all'
+    assert_response :success
+    assert_equal Role.sorted.to_a, assigns(:roles)
+    assert_equal Tracker.sorted.to_a, assigns(:trackers)
+  end
+
   def test_get_edit_with_role_and_tracker_and_all_statuses
     WorkflowTransition.delete_all
 
@@ -90,6 +106,20 @@ class WorkflowsControllerTest < ActionController::TestCase
     assert_nil      WorkflowTransition.where(:role_id => 2, :tracker_id => 1, :old_status_id => 5, :new_status_id => 4).first
   end
 
+  def test_post_edit_with_allowed_statuses_for_new_issues
+    WorkflowTransition.delete_all
+
+    post :edit, :role_id => 2, :tracker_id => 1,
+      :transitions => {
+        '0' => {'1' => {'always' => '1'}, '2' => {'always' => '1'}}
+      }
+    assert_response 302
+
+    assert WorkflowTransition.where(:role_id => 2, :tracker_id => 1, :old_status_id => 0, :new_status_id => 1).any?
+    assert WorkflowTransition.where(:role_id => 2, :tracker_id => 1, :old_status_id => 0, :new_status_id => 2).any?
+    assert_equal 2, WorkflowTransition.where(:tracker_id => 1, :role_id => 2).count
+  end
+
   def test_post_edit_with_additional_transitions
     WorkflowTransition.delete_all
   
@@ -135,13 +165,13 @@ class WorkflowsControllerTest < ActionController::TestCase
     assert_response :success
     assert_template 'permissions'
 
-    assert_select 'input[name=?][value=1]', 'role_id[]'
-    assert_select 'input[name=?][value=2]', 'tracker_id[]'
+    assert_select 'input[name=?][value="1"]', 'role_id[]'
+    assert_select 'input[name=?][value="2"]', 'tracker_id[]'
 
     # Required field
     assert_select 'select[name=?]', 'permissions[2][assigned_to_id]' do
-      assert_select 'option[value=]'
-      assert_select 'option[value=][selected=selected]', 0
+      assert_select 'option[value=""]'
+      assert_select 'option[value=""][selected=selected]', 0
       assert_select 'option[value=readonly]', :text => 'Read-only'
       assert_select 'option[value=readonly][selected=selected]', 0
       assert_select 'option[value=required]', :text => 'Required'
@@ -150,8 +180,8 @@ class WorkflowsControllerTest < ActionController::TestCase
 
     # Read-only field
     assert_select 'select[name=?]', 'permissions[3][fixed_version_id]' do
-      assert_select 'option[value=]'
-      assert_select 'option[value=][selected=selected]', 0
+      assert_select 'option[value=""]'
+      assert_select 'option[value=""][selected=selected]', 0
       assert_select 'option[value=readonly]', :text => 'Read-only'
       assert_select 'option[value=readonly][selected=selected]'
       assert_select 'option[value=required]', :text => 'Required'
@@ -160,8 +190,8 @@ class WorkflowsControllerTest < ActionController::TestCase
 
     # Other field
     assert_select 'select[name=?]', 'permissions[3][due_date]' do
-      assert_select 'option[value=]'
-      assert_select 'option[value=][selected=selected]', 0
+      assert_select 'option[value=""]'
+      assert_select 'option[value=""][selected=selected]', 0
       assert_select 'option[value=readonly]', :text => 'Read-only'
       assert_select 'option[value=readonly][selected=selected]', 0
       assert_select 'option[value=required]', :text => 'Required'
@@ -179,7 +209,7 @@ class WorkflowsControllerTest < ActionController::TestCase
     # Custom field that is always required
     # The default option is "(Required)"
     assert_select 'select[name=?]', "permissions[3][#{cf.id}]" do
-      assert_select 'option[value=]'
+      assert_select 'option[value=""]'
       assert_select 'option[value=readonly]', :text => 'Read-only'
       assert_select 'option[value=required]', 0
     end
@@ -198,7 +228,7 @@ class WorkflowsControllerTest < ActionController::TestCase
     assert_select 'select[name=?]:not(.disabled)', "permissions[1][#{cf3.id}]"
 
     assert_select 'select[name=?][disabled=disabled]', "permissions[1][#{cf2.id}]" do
-      assert_select 'option[value=][selected=selected]', :text => 'Hidden'
+      assert_select 'option[value=""][selected=selected]', :text => 'Hidden'
     end
   end
 
@@ -251,6 +281,15 @@ class WorkflowsControllerTest < ActionController::TestCase
     assert_equal IssueStatus.sorted.to_a, assigns(:statuses)
   end
 
+  def test_get_permissions_should_set_css_class
+    WorkflowPermission.delete_all
+    WorkflowPermission.create!(:role_id => 1, :tracker_id => 2, :old_status_id => 1, :field_name => 'assigned_to_id', :rule => 'required')
+
+    get :permissions, :role_id => 1, :tracker_id => 2
+    assert_response :success
+    assert_select 'td.required > select[name=?]', 'permissions[1][assigned_to_id]'
+  end
+
   def test_post_permissions
     WorkflowPermission.delete_all
 
@@ -277,16 +316,16 @@ class WorkflowsControllerTest < ActionController::TestCase
     assert_response :success
     assert_template 'copy'
     assert_select 'select[name=source_tracker_id]' do
-      assert_select 'option[value=1]', :text => 'Bug'
+      assert_select 'option[value="1"]', :text => 'Bug'
     end
     assert_select 'select[name=source_role_id]' do
-      assert_select 'option[value=2]', :text => 'Developer'
+      assert_select 'option[value="2"]', :text => 'Developer'
     end
     assert_select 'select[name=?]', 'target_tracker_ids[]' do
-      assert_select 'option[value=3]', :text => 'Support request'
+      assert_select 'option[value="3"]', :text => 'Support request'
     end
     assert_select 'select[name=?]', 'target_role_ids[]' do
-      assert_select 'option[value=1]', :text => 'Manager'
+      assert_select 'option[value="1"]', :text => 'Manager'
     end
   end
 
