@@ -1,5 +1,5 @@
 # Redmine - project management software
-# Copyright (C) 2006-2014  Jean-Philippe Lang
+# Copyright (C) 2006-2016  Jean-Philippe Lang
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -17,48 +17,60 @@
 
 require File.expand_path('../../test_helper', __FILE__)
 
-class AccountTest < ActionDispatch::IntegrationTest
-  fixtures :users, :roles
+class AccountTest < Redmine::IntegrationTest
+  fixtures :users, :email_addresses, :roles
 
   def test_login
-    get "my/page"
+    get "/my/page"
     assert_redirected_to "/login?back_url=http%3A%2F%2Fwww.example.com%2Fmy%2Fpage"
     log_user('jsmith', 'jsmith')
 
-    get "my/account"
+    get "/my/account"
     assert_response :success
     assert_template "my/account"
   end
 
+  def test_login_should_set_session_token
+    assert_difference 'Token.count' do
+      log_user('jsmith', 'jsmith')
+
+      assert_equal 2, session[:user_id]
+      assert_not_nil session[:tk]
+    end
+  end
+
   def test_autologin
     user = User.find(1)
-    Setting.autologin = "7"
     Token.delete_all
 
-    # User logs in with 'autologin' checked
-    post '/login', :username => user.login, :password => 'admin', :autologin => 1
-    assert_redirected_to '/my/page'
-    token = Token.first
-    assert_not_nil token
-    assert_equal user, token.user
-    assert_equal 'autologin', token.action
-    assert_equal user.id, session[:user_id]
-    assert_equal token.value, cookies['autologin']
-
-    # Session is cleared
-    reset!
-    User.current = nil
-    # Clears user's last login timestamp
-    user.update_attribute :last_login_on, nil
-    assert_nil user.reload.last_login_on
-
-    # User comes back with user's autologin cookie
-    cookies[:autologin] = token.value
-    get '/my/page'
-    assert_response :success
-    assert_template 'my/page'
-    assert_equal user.id, session[:user_id]
-    assert_not_nil user.reload.last_login_on
+    with_settings :autologin => '7' do
+      assert_difference 'Token.count', 2 do
+        # User logs in with 'autologin' checked
+        post '/login', :username => user.login, :password => 'admin', :autologin => 1
+        assert_redirected_to '/my/page'
+      end
+      token = Token.where(:action => 'autologin').order(:id => :desc).first
+      assert_not_nil token
+      assert_equal user, token.user
+      assert_equal 'autologin', token.action
+      assert_equal user.id, session[:user_id]
+      assert_equal token.value, cookies['autologin']
+  
+      # Session is cleared
+      reset!
+      User.current = nil
+      # Clears user's last login timestamp
+      user.update_attribute :last_login_on, nil
+      assert_nil user.reload.last_login_on
+  
+      # User comes back with user's autologin cookie
+      cookies[:autologin] = token.value
+      get '/my/page'
+      assert_response :success
+      assert_template 'my/page'
+      assert_equal user.id, session[:user_id]
+      assert_not_nil user.reload.last_login_on
+    end
   end
 
   def test_autologin_should_use_autologin_cookie_name
@@ -66,12 +78,13 @@ class AccountTest < ActionDispatch::IntegrationTest
     Redmine::Configuration.stubs(:[]).with('autologin_cookie_name').returns('custom_autologin')
     Redmine::Configuration.stubs(:[]).with('autologin_cookie_path').returns('/')
     Redmine::Configuration.stubs(:[]).with('autologin_cookie_secure').returns(false)
+    Redmine::Configuration.stubs(:[]).with('sudo_mode_timeout').returns(15)
 
     with_settings :autologin => '7' do
-      assert_difference 'Token.count' do
+      assert_difference 'Token.count', 2 do
         post '/login', :username => 'admin', :password => 'admin', :autologin => 1
+        assert_response 302
       end
-      assert_response 302
       assert cookies['custom_autologin'].present?
       token = cookies['custom_autologin']
 
@@ -81,7 +94,7 @@ class AccountTest < ActionDispatch::IntegrationTest
       get '/my/page'
       assert_response :success
 
-      assert_difference 'Token.count', -1 do
+      assert_difference 'Token.count', -2 do
         post '/logout'
       end
       assert cookies['custom_autologin'].blank?
@@ -91,12 +104,12 @@ class AccountTest < ActionDispatch::IntegrationTest
   def test_lost_password
     Token.delete_all
 
-    get "account/lost_password"
+    get "/account/lost_password"
     assert_response :success
     assert_template "account/lost_password"
     assert_select 'input[name=mail]'
 
-    post "account/lost_password", :mail => 'jSmith@somenet.foo'
+    post "/account/lost_password", :mail => 'jSmith@somenet.foo'
     assert_redirected_to "/login"
 
     token = Token.first
@@ -104,21 +117,21 @@ class AccountTest < ActionDispatch::IntegrationTest
     assert_equal 'jsmith@somenet.foo', token.user.mail
     assert !token.expired?
 
-    get "account/lost_password", :token => token.value
+    get "/account/lost_password", :token => token.value
     assert_response :success
     assert_template "account/password_recovery"
     assert_select 'input[type=hidden][name=token][value=?]', token.value
     assert_select 'input[name=new_password]'
     assert_select 'input[name=new_password_confirmation]'
 
-    post "account/lost_password",
+    post "/account/lost_password",
          :token => token.value, :new_password => 'newpass123',
          :new_password_confirmation => 'newpass123'
     assert_redirected_to "/login"
     assert_equal 'Password was successfully updated.', flash[:notice]
 
     log_user('jsmith', 'newpass123')
-    assert_equal 0, Token.count
+    assert_equal false, Token.exists?(token.id), "Password recovery token was not deleted"
   end
 
   def test_user_with_must_change_passwd_should_be_forced_to_change_its_password
@@ -150,14 +163,48 @@ class AccountTest < ActionDispatch::IntegrationTest
     assert_equal false, User.find_by_login('jsmith').must_change_passwd?
   end
 
+  def test_user_with_expired_password_should_be_forced_to_change_its_password
+    User.find_by_login('jsmith').update_attribute :passwd_changed_on, 14.days.ago
+
+    with_settings :password_max_age => 7 do
+      post '/login', :username => 'jsmith', :password => 'jsmith'
+      assert_redirected_to '/my/page'
+      follow_redirect!
+      assert_redirected_to '/my/password'
+
+      get '/issues'
+      assert_redirected_to '/my/password'
+    end
+  end
+
+  def test_user_with_expired_password_should_be_able_to_change_its_password
+    User.find_by_login('jsmith').update_attribute :passwd_changed_on, 14.days.ago
+
+    with_settings :password_max_age => 7 do
+      post '/login', :username => 'jsmith', :password => 'jsmith'
+      assert_redirected_to '/my/page'
+      follow_redirect!
+      assert_redirected_to '/my/password'
+      follow_redirect!
+      assert_response :success
+      post '/my/password', :password => 'jsmith', :new_password => 'newpassword', :new_password_confirmation => 'newpassword'
+      assert_redirected_to '/my/account'
+      follow_redirect!
+      assert_response :success
+
+      assert_equal false, User.find_by_login('jsmith').must_change_passwd?
+    end
+
+  end
+
   def test_register_with_automatic_activation
     Setting.self_registration = '3'
 
-    get 'account/register'
+    get '/account/register'
     assert_response :success
     assert_template 'account/register'
 
-    post 'account/register',
+    post '/account/register',
          :user => {:login => "newuser", :language => "en",
                    :firstname => "New", :lastname => "User", :mail => "newuser@foo.bar",
                    :password => "newpass123", :password_confirmation => "newpass123"}
@@ -175,7 +222,7 @@ class AccountTest < ActionDispatch::IntegrationTest
   def test_register_with_manual_activation
     Setting.self_registration = '2'
 
-    post 'account/register',
+    post '/account/register',
          :user => {:login => "newuser", :language => "en",
                    :firstname => "New", :lastname => "User", :mail => "newuser@foo.bar",
                    :password => "newpass123", :password_confirmation => "newpass123"}
@@ -187,7 +234,7 @@ class AccountTest < ActionDispatch::IntegrationTest
     Setting.self_registration = '1'
     Token.delete_all
 
-    post 'account/register',
+    post '/account/register',
          :user => {:login => "newuser", :language => "en",
                    :firstname => "New", :lastname => "User", :mail => "newuser@foo.bar",
                    :password => "newpass123", :password_confirmation => "newpass123"}
@@ -199,7 +246,7 @@ class AccountTest < ActionDispatch::IntegrationTest
     assert_equal 'newuser@foo.bar', token.user.mail
     assert !token.expired?
 
-    get 'account/activate', :token => token.value
+    get '/account/activate', :token => token.value
     assert_redirected_to '/login'
     log_user('newuser', 'newpass123')
   end
@@ -229,12 +276,12 @@ class AccountTest < ActionDispatch::IntegrationTest
     post '/login', :username => 'foo', :password => 'bar'
     assert_response :success
     assert_template 'account/register'
-    assert_tag :input, :attributes => { :name => 'user[firstname]', :value => '' }
-    assert_tag :input, :attributes => { :name => 'user[lastname]', :value => 'Smith' }
-    assert_no_tag :input, :attributes => { :name => 'user[login]' }
-    assert_no_tag :input, :attributes => { :name => 'user[password]' }
+    assert_select 'input[name=?][value=""]', 'user[firstname]'
+    assert_select 'input[name=?][value=Smith]', 'user[lastname]'
+    assert_select 'input[name=?]', 'user[login]', 0
+    assert_select 'input[name=?]', 'user[password]', 0
 
-    post 'account/register',
+    post '/account/register',
          :user => {:firstname => 'Foo', :lastname => 'Smith', :mail => 'foo@bar.com'}
     assert_redirected_to '/my/account'
 
@@ -251,7 +298,7 @@ class AccountTest < ActionDispatch::IntegrationTest
       # register a new account
       assert_difference 'User.count' do
         assert_difference 'Token.count' do
-          post 'account/register',
+          post '/account/register',
              :user => {:login => "newuser", :language => "en",
                        :firstname => "New", :lastname => "User", :mail => "newuser@foo.bar",
                        :password => "newpass123", :password_confirmation => "newpass123"}
@@ -269,7 +316,7 @@ class AccountTest < ActionDispatch::IntegrationTest
       follow_redirect!
       assert_response :success
       assert_select 'div.flash', :text => /new activation email/
-      assert_select 'div.flash a[href=/account/activation_email]'
+      assert_select 'div.flash a[href="/account/activation_email"]'
 
       # request a new action activation email
       assert_difference 'ActionMailer::Base.deliveries.size' do
